# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_display_aio.py 71793 2026-02-03 18:07:40Z mvuilleu $
#
#  Implements the asyncio YDisplay API for Display functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YDisplay and YDisplayLayer
version: PATCH_WITH_VERSION
requires: yocto_api_aio
provides: YDisplay YDisplayLayer
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final, Any
    from collections.abc import Callable, Awaitable
    from enum import IntEnum
    from .yocto_api_aio import _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, YFunction, HwId, hwid2str,
    xarray, xbytearray, xStringIO
)


# noinspection PyProtectedMember
class YDisplayLayer:
    """
    Each DisplayLayer represents an image layer containing objects
    to display (bitmaps, text, etc.). The content is displayed only when
    the layer is active on the screen (and not masked by other
    overlapping layers).

    """

    # --- (end of generated code: YDisplayLayer class start)

    # --- (generated code: YDisplayLayer attributes declaration)
    _polyPrevX: int
    _polyPrevY: int
    # --- (end of generated code: YDisplayLayer attributes declaration)

    if not _IS_MICROPYTHON:
        # --- (generated code: YDisplayLayer return codes)
        NO_INK: Final[int] = -1
        BG_INK: Final[int] = -2
        FG_INK: Final[int] = -3
        class ALIGN(IntEnum):
            TOP_LEFT = 0
            CENTER_LEFT = 1
            BASELINE_LEFT = 2
            BOTTOM_LEFT = 3
            TOP_CENTER = 4
            CENTER = 5
            BASELINE_CENTER = 6
            BOTTOM_CENTER = 7
            TOP_DECIMAL = 8
            CENTER_DECIMAL = 9
            BASELINE_DECIMAL = 10
            BOTTOM_DECIMAL = 11
            TOP_RIGHT = 12
            CENTER_RIGHT = 13
            BASELINE_RIGHT = 14
            BOTTOM_RIGHT = 15

        # --- (end of generated code: YDisplayLayer return codes)

    def __init__(self, parent, layerId):
        self._display = parent
        self._id = int(layerId)
        self._cmdbuff = ""
        self._hidden = False
        # --- (generated code: YDisplayLayer constructor)
        self._polyPrevX = 0
        self._polyPrevY = 0
        # --- (end of generated code: YDisplayLayer constructor)

    async def flush_now(self) -> int:
        res = YAPI.SUCCESS
        if self._cmdbuff != "":
            res = await self._display.sendCommand(self._cmdbuff)
            self._cmdbuff = ""
        return res

    async def command_push(self, cmd) -> int:
        res = YAPI.SUCCESS
        if len(self._cmdbuff) + len(cmd) >= 100:
            res = await self.flush_now()
        if self._cmdbuff == "":
            self._cmdbuff = str(self._id)
        self._cmdbuff = self._cmdbuff + cmd
        return res

    async def command_flush(self, cmd) -> int:
        res = await self.command_push(cmd)
        if not self._hidden:
            res = await self.flush_now()
        return res

    # --- (generated code: YDisplayLayer implementation)
    async def reset(self) -> int:
        """
        Reverts the layer to its initial state (fully transparent, default settings).
        Reinitializes the drawing pointer to the upper left position,
        and selects the most visible pen color. If you only want to erase the layer
        content, use the method clear() instead.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        self._hidden = False
        return await self.command_flush("X")

    async def clear(self) -> int:
        """
        Erases the whole content of the layer (makes it fully transparent).
        This method does not change any other attribute of the layer.
        To reinitialize the layer attributes to defaults settings, use the method
        reset() instead.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("x")

    async def selectColorPen(self, color: int) -> int:
        """
        Selects the color to be used for all subsequent drawing functions,
        for filling as well as for line and text drawing.
        To select a different fill and outline color, use
        selectFillColor and selectLineColor.
        The pen color is provided as an RGB value.
        For grayscale or monochrome displays, the value is
        automatically converted to the proper range.

        @param color : the desired pen color, as a 24-bit RGB value

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("c%06x" % color)

    async def selectGrayPen(self, graylevel: int) -> int:
        """
        Selects the pen gray level for all subsequent drawing functions,
        for filling as well as for line and text drawing.
        To select a different fill and outline color, use
        selectFillColor and selectLineColor.
        The gray level is provided as a number between
        0 (black) and 255 (white, or whichever the lightest color is).
        For monochrome displays (without gray levels), any value
        lower than 128 is rendered as black, and any value equal
        or above to 128 is non-black.

        @param graylevel : the desired gray level, from 0 to 255

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("g%d" % graylevel)

    async def selectEraser(self) -> int:
        """
        Selects an eraser instead of a pen for all subsequent drawing functions,
        except for bitmap copy functions. Any point drawn using the eraser
        becomes transparent (as when the layer is empty), showing the other
        layers beneath it.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("e")

    async def selectFillColor(self, color: int) -> int:
        """
        Selects the color to be used for filling rectangular bars,
        discs and polygons. The color is provided as an RGB value.
        For grayscale or monochrome displays, the value is
        automatically converted to the proper range.
        You can also use the constants FG_INK to use the
        default drawing colour, BG_INK to use the default
        background colour, and NO_INK to disable filling.

        @param color : the desired drawing color, as a 24-bit RGB value,
                or one of the constants NO_INK, FG_INK
                or BG_INK

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        r: int
        g: int
        b: int
        if color==-1:
            return await self.command_push("f_")
        if color==-2:
            return await self.command_push("f-")
        if color==-3:
            return await self.command_push("f.")
        r = ((color >> 20) & 15)
        g = ((color >> 12) & 15)
        b = ((color >> 4) & 15)
        return await self.command_push("f%x%x%x" % (r, g, b))

    async def selectLineColor(self, color: int) -> int:
        """
        Selects the color to be used for drawing the outline of rectangular
        bars, discs and polygons, as well as for drawing lines and text.
        The color is provided as an RGB value.
        For grayscale or monochrome displays, the value is
        automatically converted to the proper range.
        You can also use the constants FG_INK to use the
        default drawing colour, BG_INK to use the default
        background colour, and NO_INK to disable outline drawing.

        @param color : the desired drawing color, as a 24-bit RGB value,
                or one of the constants NO_INK, FG_INK
                or BG_INK

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        r: int
        g: int
        b: int
        if color==-1:
            return await self.command_push("l_")
        if color==-2:
            return await self.command_push("l-")
        if color==-3:
            return await self.command_push("l*")
        r = ((color >> 20) & 15)
        g = ((color >> 12) & 15)
        b = ((color >> 4) & 15)
        return await self.command_push("l%x%x%x" % (r, g, b))

    async def selectLineWidth(self, width: int) -> int:
        """
        Selects the line width for drawing the outline of rectangular
        bars, discs and polygons, as well as for drawing lines.

        @param width : the desired line width, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("t%d" % width)

    async def setAntialiasingMode(self, mode: bool) -> int:
        return await self.command_push("a%d" % mode)

    async def drawPixel(self, x: int, y: int) -> int:
        """
        Draws a single pixel at the specified position.

        @param x : the distance from left of layer, in pixels
        @param y : the distance from top of layer, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("P%d,%d" % (x, y))

    async def drawRect(self, x1: int, y1: int, x2: int, y2: int) -> int:
        """
        Draws an empty rectangle at a specified position.

        @param x1 : the distance from left of layer to the left border of the rectangle, in pixels
        @param y1 : the distance from top of layer to the top border of the rectangle, in pixels
        @param x2 : the distance from left of layer to the right border of the rectangle, in pixels
        @param y2 : the distance from top of layer to the bottom border of the rectangle, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("R%d,%d,%d,%d" % (x1, y1, x2, y2))

    async def drawBar(self, x1: int, y1: int, x2: int, y2: int) -> int:
        """
        Draws a filled rectangular bar at a specified position.

        @param x1 : the distance from left of layer to the left border of the rectangle, in pixels
        @param y1 : the distance from top of layer to the top border of the rectangle, in pixels
        @param x2 : the distance from left of layer to the right border of the rectangle, in pixels
        @param y2 : the distance from top of layer to the bottom border of the rectangle, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("B%d,%d,%d,%d" % (x1, y1, x2, y2))

    async def drawCircle(self, x: int, y: int, r: int) -> int:
        """
        Draws an empty circle at a specified position.

        @param x : the distance from left of layer to the center of the circle, in pixels
        @param y : the distance from top of layer to the center of the circle, in pixels
        @param r : the radius of the circle, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("C%d,%d,%d" % (x, y, r))

    async def drawDisc(self, x: int, y: int, r: int) -> int:
        """
        Draws a filled disc at a given position.

        @param x : the distance from left of layer to the center of the disc, in pixels
        @param y : the distance from top of layer to the center of the disc, in pixels
        @param r : the radius of the disc, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("D%d,%d,%d" % (x, y, r))

    async def selectFont(self, fontname: str) -> int:
        """
        Selects a font to use for the next text drawing functions, by providing the name of the
        font file. You can use a built-in font as well as a font file that you have previously
        uploaded to the device built-in memory. If you experience problems selecting a font
        file, check the device logs for any error message such as missing font file or bad font
        file format.

        @param fontname : the font file name, embedded fonts are 8x8.yfm, Small.yfm, Medium.yfm, Large.yfm
        (not available on Yocto-MiniDisplay).

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("&%s%c" % (fontname, 27))

    async def drawText(self, x: int, y: int, anchor: ALIGN, text: str) -> int:
        """
        Draws a text string at the specified position. The point of the text that is aligned
        to the specified pixel position is called the anchor point, and can be chosen among
        several options. Text is rendered from left to right, without implicit wrapping.

        @param x : the distance from left of layer to the text anchor point, in pixels
        @param y : the distance from top of layer to the text anchor point, in pixels
        @param anchor : the text anchor point, chosen among the YDisplayLayer.ALIGN enumeration:
                YDisplayLayer.ALIGN.TOP_LEFT,         YDisplayLayer.ALIGN.CENTER_LEFT,
                YDisplayLayer.ALIGN.BASELINE_LEFT,    YDisplayLayer.ALIGN.BOTTOM_LEFT,
                YDisplayLayer.ALIGN.TOP_CENTER,       YDisplayLayer.ALIGN.CENTER,
                YDisplayLayer.ALIGN.BASELINE_CENTER,  YDisplayLayer.ALIGN.BOTTOM_CENTER,
                YDisplayLayer.ALIGN.TOP_DECIMAL,      YDisplayLayer.ALIGN.CENTER_DECIMAL,
                YDisplayLayer.ALIGN.BASELINE_DECIMAL, YDisplayLayer.ALIGN.BOTTOM_DECIMAL,
                YDisplayLayer.ALIGN.TOP_RIGHT,        YDisplayLayer.ALIGN.CENTER_RIGHT,
                YDisplayLayer.ALIGN.BASELINE_RIGHT,   YDisplayLayer.ALIGN.BOTTOM_RIGHT.
        @param text : the text string to draw

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("T%d,%d,%d,%s%c" % (x, y, anchor, text, 27))

    async def drawImage(self, x: int, y: int, imagename: str) -> int:
        """
        Draws an image previously uploaded to the device filesystem, at the specified position.
        At present time, GIF images are the only supported image format. If you experience
        problems using an image file, check the device logs for any error message such as
        missing image file or bad image file format.

        @param x : the distance from left of layer to the left of the image, in pixels
        @param y : the distance from top of layer to the top of the image, in pixels
        @param imagename : the GIF file name

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("*%d,%d,%s%c" % (x, y, imagename, 27))

    async def drawBitmap(self, x: int, y: int, w: int, bitmap: xarray, bgcol: int) -> int:
        """
        Draws a bitmap at the specified position. The bitmap is provided as a binary object,
        where each pixel maps to a bit, from left to right and from top to bottom.
        The most significant bit of each byte maps to the leftmost pixel, and the least
        significant bit maps to the rightmost pixel. Bits set to 1 are drawn using the
        layer selected pen color. Bits set to 0 are drawn using the specified background
        gray level, unless -1 is specified, in which case they are not drawn at all
        (as if transparent).

        @param x : the distance from left of layer to the left of the bitmap, in pixels
        @param y : the distance from top of layer to the top of the bitmap, in pixels
        @param w : the width of the bitmap, in pixels
        @param bitmap : a binary object
        @param bgcol : the background gray level to use for zero bits (0 = black,
                255 = white), or -1 to leave the pixels unchanged

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        destname: str
        destname = "layer%d:%d,%d@%d,%d" % (self._id, w, bgcol, x, y)
        return await self._display.upload(destname, bitmap)

    async def drawGIF(self, x: int, y: int, gifimage: xarray) -> int:
        """
        Draws a GIF image provided as a binary buffer at the specified position.
        If the image drawing must be included in an animation sequence, save it
        in the device filesystem first and use drawImage instead.

        @param x : the distance from left of layer to the left of the image, in pixels
        @param y : the distance from top of layer to the top of the image, in pixels
        @param gifimage : a binary object with the content of a GIF file

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        destname: str
        destname = "layer%d:G,-1@%d,%d" % (self._id, x, y)
        return await self._display.upload(destname, gifimage)

    async def moveTo(self, x: int, y: int) -> int:
        """
        Moves the drawing pointer of this layer to the specified position.

        @param x : the distance from left of layer, in pixels
        @param y : the distance from top of layer, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("@%d,%d" % (x, y))

    async def lineTo(self, x: int, y: int) -> int:
        """
        Draws a line from current drawing pointer position to the specified position.
        The specified destination pixel is included in the line. The pointer position
        is then moved to the end point of the line.

        @param x : the distance from left of layer to the end point of the line, in pixels
        @param y : the distance from top of layer to the end point of the line, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("-%d,%d" % (x, y))

    async def polygonStart(self, x: int, y: int) -> int:
        """
        Starts drawing a polygon with the first corner at the specified position.

        @param x : the distance from left of layer, in pixels
        @param y : the distance from top of layer, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        self._polyPrevX = x
        self._polyPrevY = y
        return await self.command_push("[%d,%d" % (x, y))

    async def polygonAdd(self, x: int, y: int) -> int:
        """
        Adds a point to the currently open polygon, previously opened using
        polygonStart.

        @param x : the distance from left of layer to the new point, in pixels
        @param y : the distance from top of layer to the new point, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        dx: int
        dy: int
        dx = x - self._polyPrevX
        dy = y - self._polyPrevY
        self._polyPrevX = x
        self._polyPrevY = y
        return await self.command_flush(";%d,%d" % (dx, dy))

    async def polygonEnd(self) -> int:
        """
        Close the currently open polygon, fill its content the fill color currently
        selected for the layer, and draw its outline using the selected line color.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("]")

    async def consoleOut(self, text: str) -> int:
        """
        Outputs a message in the console area, and advances the console pointer accordingly.
        The console pointer position is automatically moved to the beginning
        of the next line when a newline character is met, or when the right margin
        is hit. When the new text to display extends below the lower margin, the
        console area is automatically scrolled up.

        @param text : the message to display

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("!%s%c" % (text, 27))

    async def setConsoleMargins(self, x1: int, y1: int, x2: int, y2: int) -> int:
        """
        Sets up display margins for the consoleOut function.

        @param x1 : the distance from left of layer to the left margin, in pixels
        @param y1 : the distance from top of layer to the top margin, in pixels
        @param x2 : the distance from left of layer to the right margin, in pixels
        @param y2 : the distance from top of layer to the bottom margin, in pixels

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("m%d,%d,%d,%d" % (x1, y1, x2, y2))

    async def setConsoleBackground(self, bgcol: int) -> int:
        """
        Sets up the background color used by the clearConsole function and by
        the console scrolling feature.

        @param bgcol : the background gray level to use when scrolling (0 = black,
                255 = white), or -1 for transparent

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("b%d" % bgcol)

    async def setConsoleWordWrap(self, wordwrap: bool) -> int:
        """
        Sets up the wrapping behavior used by the consoleOut function.

        @param wordwrap : true to wrap only between words,
                false to wrap on the last column anyway.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_push("w%d" % wordwrap)

    async def clearConsole(self) -> int:
        """
        Blanks the console area within console margins, and resets the console pointer
        to the upper left corner of the console.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("^")

    async def setLayerPosition(self, x: int, y: int, scrollTime: int) -> int:
        """
        Sets the position of the layer relative to the display upper left corner.
        When smooth scrolling is used, the display offset of the layer is
        automatically updated during the next milliseconds to animate the move of the layer.

        @param x : the distance from left of display to the upper left corner of the layer
        @param y : the distance from top of display to the upper left corner of the layer
        @param scrollTime : number of milliseconds to use for smooth scrolling, or
                0 if the scrolling should be immediate.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.command_flush("#%d,%d,%d" % (x, y, scrollTime))

    async def hide(self) -> int:
        """
        Hides the layer. The state of the layer is preserved but the layer is not displayed
        on the screen until the next call to unhide(). Hiding the layer can positively
        affect the drawing speed, since it postpones the rendering until all operations are
        completed (double-buffering).

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.command_push("h")
        self._hidden = True
        return await self.flush_now()

    async def unhide(self) -> int:
        """
        Shows the layer. Shows the layer again after a hide command.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        self._hidden = False
        return await self.command_flush("s")

    async def get_display(self) -> YDisplay:
        """
        Gets parent YDisplay. Returns the parent YDisplay object of the current YDisplayLayer.

        @return an YDisplay object
        """
        return self._display

    async def get_displayWidth(self) -> int:
        """
        Returns the display width, in pixels.

        @return an integer corresponding to the display width, in pixels

        On failure, throws an exception or returns YDisplayLayer.DISPLAYWIDTH_INVALID.
        """
        return await self._display.get_displayWidth()

    async def get_displayHeight(self) -> int:
        """
        Returns the display height, in pixels.

        @return an integer corresponding to the display height, in pixels

        On failure, throws an exception or returns YDisplayLayer.DISPLAYHEIGHT_INVALID.
        """
        return await self._display.get_displayHeight()

    async def get_layerWidth(self) -> int:
        """
        Returns the width of the layers to draw on, in pixels.

        @return an integer corresponding to the width of the layers to draw on, in pixels

        On failure, throws an exception or returns YDisplayLayer.LAYERWIDTH_INVALID.
        """
        return await self._display.get_layerWidth()

    async def get_layerHeight(self) -> int:
        """
        Returns the height of the layers to draw on, in pixels.

        @return an integer corresponding to the height of the layers to draw on, in pixels

        On failure, throws an exception or returns YDisplayLayer.LAYERHEIGHT_INVALID.
        """
        return await self._display.get_layerHeight()

    async def resetHiddenFlag(self) -> int:
        self._hidden = False
        return YAPI.SUCCESS

    # --- (end of generated code: YDisplayLayer implementation)


# --- (generated code: YDisplayLayer functions)
# --- (end of generated code: YDisplayLayer functions)


# --- (generated code: YDisplay class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YDisplayValueCallback = Union[Callable[['YDisplay', str], Any], None]
    except TypeError:
        YDisplayValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YDisplay(YFunction):
    """
    The YDisplay class allows to drive Yoctopuce displays.
    Yoctopuce display interface has been designed to easily
    show information and images. The device provides built-in
    multi-layer rendering. Layers can be drawn offline, individually,
    and freely moved on the display. It can also replay recorded
    sequences (animations).

    In order to draw on the screen, you should use the
    display.get_displayLayer method to retrieve the layer(s) on
    which you want to draw, and then use methods defined in
    YDisplayLayer to draw on the layers.

    """
    # --- (end of generated code: YDisplay class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YDisplay return codes)
        STARTUPSEQ_INVALID: Final[str] = YAPI.INVALID_STRING
        BRIGHTNESS_INVALID: Final[int] = YAPI.INVALID_UINT
        AUTOINVERTDELAY_INVALID: Final[int] = YAPI.INVALID_UINT
        DISPLAYPANEL_INVALID: Final[str] = YAPI.INVALID_STRING
        DISPLAYWIDTH_INVALID: Final[int] = YAPI.INVALID_UINT
        DISPLAYHEIGHT_INVALID: Final[int] = YAPI.INVALID_UINT
        LAYERWIDTH_INVALID: Final[int] = YAPI.INVALID_UINT
        LAYERHEIGHT_INVALID: Final[int] = YAPI.INVALID_UINT
        LAYERCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        ENABLED_FALSE: Final[int] = 0
        ENABLED_TRUE: Final[int] = 1
        ENABLED_INVALID: Final[int] = -1
        ORIENTATION_LEFT: Final[int] = 0
        ORIENTATION_UP: Final[int] = 1
        ORIENTATION_RIGHT: Final[int] = 2
        ORIENTATION_DOWN: Final[int] = 3
        ORIENTATION_INVALID: Final[int] = -1
        DISPLAYTYPE_MONO: Final[int] = 0
        DISPLAYTYPE_EPAPER_BW: Final[int] = 1
        DISPLAYTYPE_EPAPER_BWR: Final[int] = 2
        DISPLAYTYPE_EPAPER_BWRY: Final[int] = 3
        DISPLAYTYPE_INVALID: Final[int] = -1
        # --- (end of generated code: YDisplay return codes)
    _sequence: str
    _recording: bool
    # --- (generated code: YDisplay attributes declaration)
    _valueCallback: YDisplayValueCallback
    _allDisplayLayers: list[YDisplayLayer]
    # --- (end of generated code: YDisplay attributes declaration)

    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, 'Display', func)
        # --- (generated code: YDisplay constructor)
        self._allDisplayLayers = []
        # --- (end of generated code: YDisplay constructor)
        self._sequence = ""
        self._recording = False

    # --- (generated code: YDisplay implementation)
    @classmethod
    def FindDisplay(cls, func: str) -> YDisplay:
        """
        Retrieves a display for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the display is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDisplay.isOnline() to test if the display is
        indeed online at a given time. In case of ambiguity when looking for
        a display by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the display, for instance
                YD128X32.display.

        @return a YDisplay object allowing you to drive the display.
        """
        return cls.FindDisplayInContext(YAPI, func)

    @classmethod
    def FindDisplayInContext(cls, yctx: YAPIContext, func: str) -> YDisplay:
        """
        Retrieves a display for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the display is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDisplay.isOnline() to test if the display is
        indeed online at a given time. In case of ambiguity when looking for
        a display by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the display, for instance
                YD128X32.display.

        @return a YDisplay object allowing you to drive the display.
        """
        obj: Union[YDisplay, None] = yctx._findInCache('Display', func)
        if obj:
            return obj
        return YDisplay(yctx, func)

    @classmethod
    def FirstDisplay(cls) -> Union[YDisplay, None]:
        """
        Starts the enumeration of displays currently accessible.
        Use the method YDisplay.nextDisplay() to iterate on
        next displays.

        @return a pointer to a YDisplay object, corresponding to
                the first display currently online, or a None pointer
                if there are none.
        """
        return cls.FirstDisplayInContext(YAPI)

    @classmethod
    def FirstDisplayInContext(cls, yctx: YAPIContext) -> Union[YDisplay, None]:
        """
        Starts the enumeration of displays currently accessible.
        Use the method YDisplay.nextDisplay() to iterate on
        next displays.

        @param yctx : a YAPI context.

        @return a pointer to a YDisplay object, corresponding to
                the first display currently online, or a None pointer
                if there are none.
        """
        hwid: Union[HwId, None] = yctx._firstHwId('Display')
        if hwid:
            return cls.FindDisplayInContext(yctx, hwid2str(hwid))
        return None

    def nextDisplay(self) -> Union[YDisplay, None]:
        """
        Continues the enumeration of displays started using yFirstDisplay().
        Caution: You can't make any assumption about the returned displays order.
        If you want to find a specific a display, use Display.findDisplay()
        and a hardwareID or a logical name.

        @return a pointer to a YDisplay object, corresponding to
                a display currently online, or a None pointer
                if there are no more displays to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            next_hwid = self._yapi._nextHwId('Display', self.get_hwId())
        except YAPI_Exception:
            pass
        if next_hwid:
            return self.FindDisplayInContext(self._yapi, hwid2str(next_hwid))
        return None

    async def get_enabled(self) -> int:
        """
        Returns true if the screen is powered, false otherwise.

        @return either YDisplay.ENABLED_FALSE or YDisplay.ENABLED_TRUE, according to true if the screen is
        powered, false otherwise

        On failure, throws an exception or returns YDisplay.ENABLED_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("enabled")
        if json_val is None:
            return YDisplay.ENABLED_INVALID
        return json_val

    async def set_enabled(self, newval: int) -> int:
        """
        Changes the power state of the display.

        @param newval : either YDisplay.ENABLED_FALSE or YDisplay.ENABLED_TRUE, according to the power
        state of the display

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = "1" if newval > 0 else "0"
        return await self._setAttr("enabled", rest_val)

    async def get_startupSeq(self) -> str:
        """
        Returns the name of the sequence to play when the displayed is powered on.

        @return a string corresponding to the name of the sequence to play when the displayed is powered on

        On failure, throws an exception or returns YDisplay.STARTUPSEQ_INVALID.
        """
        json_val: Union[str, None] = await self._fromCache("startupSeq")
        if json_val is None:
            return YDisplay.STARTUPSEQ_INVALID
        return json_val

    async def set_startupSeq(self, newval: str) -> int:
        """
        Changes the name of the sequence to play when the displayed is powered on.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the name of the sequence to play when the displayed is powered on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("startupSeq", rest_val)

    async def get_brightness(self) -> int:
        """
        Returns the luminosity of the  module informative LEDs (from 0 to 100).

        @return an integer corresponding to the luminosity of the  module informative LEDs (from 0 to 100)

        On failure, throws an exception or returns YDisplay.BRIGHTNESS_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("brightness")
        if json_val is None:
            return YDisplay.BRIGHTNESS_INVALID
        return json_val

    async def set_brightness(self, newval: int) -> int:
        """
        Changes the brightness of the display. The parameter is a value between 0 and
        100. Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the brightness of the display

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("brightness", rest_val)

    async def get_autoInvertDelay(self) -> int:
        """
        Returns the interval between automatic display inversions, or 0 if automatic
        inversion is disabled. Using the automatic inversion mechanism reduces the
        burn-in that occurs on OLED screens over long periods when the same content
        remains displayed on the screen.

        @return an integer corresponding to the interval between automatic display inversions, or 0 if automatic
                inversion is disabled

        On failure, throws an exception or returns YDisplay.AUTOINVERTDELAY_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("autoInvertDelay")
        if json_val is None:
            return YDisplay.AUTOINVERTDELAY_INVALID
        return json_val

    async def set_autoInvertDelay(self, newval: int) -> int:
        """
        Changes the interval between automatic display inversions.
        The parameter is the number of seconds, or 0 to disable automatic inversion.
        Using the automatic inversion mechanism reduces the burn-in that occurs on OLED
        screens over long periods when the same content remains displayed on the screen.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the interval between automatic display inversions

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("autoInvertDelay", rest_val)

    async def get_orientation(self) -> int:
        """
        Returns the currently selected display orientation.

        @return a value among YDisplay.ORIENTATION_LEFT, YDisplay.ORIENTATION_UP,
        YDisplay.ORIENTATION_RIGHT and YDisplay.ORIENTATION_DOWN corresponding to the currently selected
        display orientation

        On failure, throws an exception or returns YDisplay.ORIENTATION_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("orientation")
        if json_val is None:
            return YDisplay.ORIENTATION_INVALID
        return json_val

    async def set_orientation(self, newval: int) -> int:
        """
        Changes the display orientation. Remember to call the saveToFlash()
        method of the module if the modification must be kept.

        @param newval : a value among YDisplay.ORIENTATION_LEFT, YDisplay.ORIENTATION_UP,
        YDisplay.ORIENTATION_RIGHT and YDisplay.ORIENTATION_DOWN corresponding to the display orientation

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("orientation", rest_val)

    async def get_displayPanel(self) -> str:
        """
        Returns the exact model of the display panel.

        @return a string corresponding to the exact model of the display panel

        On failure, throws an exception or returns YDisplay.DISPLAYPANEL_INVALID.
        """
        json_val: Union[str, None] = await self._fromCache("displayPanel")
        if json_val is None:
            return YDisplay.DISPLAYPANEL_INVALID
        return json_val

    async def set_displayPanel(self, newval: str) -> int:
        """
        Changes the model of display to match the connected display panel.
        This function has no effect if the module does not support the selected
        display panel.
        Remember to call the saveToFlash()
        method of the module if the modification must be kept.

        @param newval : a string corresponding to the model of display to match the connected display panel

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("displayPanel", rest_val)

    async def get_displayWidth(self) -> int:
        """
        Returns the display width, in pixels.

        @return an integer corresponding to the display width, in pixels

        On failure, throws an exception or returns YDisplay.DISPLAYWIDTH_INVALID.
        """
        json_val: Union[int, None] = await self._lazyCache("displayWidth")
        if json_val is None:
            return YDisplay.DISPLAYWIDTH_INVALID
        return json_val

    async def get_displayHeight(self) -> int:
        """
        Returns the display height, in pixels.

        @return an integer corresponding to the display height, in pixels

        On failure, throws an exception or returns YDisplay.DISPLAYHEIGHT_INVALID.
        """
        json_val: Union[int, None] = await self._lazyCache("displayHeight")
        if json_val is None:
            return YDisplay.DISPLAYHEIGHT_INVALID
        return json_val

    async def get_displayType(self) -> int:
        """
        Returns the display type: monochrome OLED, black and white ePaper, color ePaper, etc.

        @return a value among YDisplay.DISPLAYTYPE_MONO, YDisplay.DISPLAYTYPE_EPAPER_BW,
        YDisplay.DISPLAYTYPE_EPAPER_BWR and YDisplay.DISPLAYTYPE_EPAPER_BWRY corresponding to the display
        type: monochrome OLED, black and white ePaper, color ePaper, etc

        On failure, throws an exception or returns YDisplay.DISPLAYTYPE_INVALID.
        """
        json_val: Union[int, None] = await self._lazyCache("displayType")
        if json_val is None:
            return YDisplay.DISPLAYTYPE_INVALID
        return json_val

    async def get_layerWidth(self) -> int:
        """
        Returns the width of the layers to draw on, in pixels.

        @return an integer corresponding to the width of the layers to draw on, in pixels

        On failure, throws an exception or returns YDisplay.LAYERWIDTH_INVALID.
        """
        json_val: Union[int, None] = await self._lazyCache("layerWidth")
        if json_val is None:
            return YDisplay.LAYERWIDTH_INVALID
        return json_val

    async def get_layerHeight(self) -> int:
        """
        Returns the height of the layers to draw on, in pixels.

        @return an integer corresponding to the height of the layers to draw on, in pixels

        On failure, throws an exception or returns YDisplay.LAYERHEIGHT_INVALID.
        """
        json_val: Union[int, None] = await self._lazyCache("layerHeight")
        if json_val is None:
            return YDisplay.LAYERHEIGHT_INVALID
        return json_val

    async def get_layerCount(self) -> int:
        """
        Returns the number of available layers to draw on.

        @return an integer corresponding to the number of available layers to draw on

        On failure, throws an exception or returns YDisplay.LAYERCOUNT_INVALID.
        """
        json_val: Union[int, None] = await self._lazyCache("layerCount")
        if json_val is None:
            return YDisplay.LAYERCOUNT_INVALID
        return json_val

    async def get_command(self) -> str:
        json_val: Union[str, None] = await self._fromCache("command")
        if json_val is None:
            return YDisplay.COMMAND_INVALID
        return json_val

    async def set_command(self, newval: str) -> int:
        rest_val = newval
        return await self._setAttr("command", rest_val)

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YDisplayValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is called once when it is registered, passing the current advertised value
            of the function, provided that it is not an empty string.
            The callback is then invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    async def resetAll(self) -> int:
        """
        Clears the display screen and resets all display layers to their default state.
        Using this function in a sequence will kill the sequence play-back. Don't use that
        function to reset the display at sequence start-up.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        await self.resetHiddenLayerFlags()
        return await self.sendCommand("Z")

    async def regenerateDisplay(self) -> int:
        """
        Forces an ePaper screen to perform a regenerative update using the slow
        update method. Periodic use of the slow method (total panel update with
        multiple inversions) prevents ghosting effects and improves contrast.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.sendCommand("z")

    async def postponeRefresh(self, duration: int) -> int:
        """
        Disables screen refresh for a short period of time. The combination of
        postponeRefresh and triggerRefresh can be used as an
        alternative to double-buffering to avoid flickering during display updates.

        @param duration : duration of deactivation in milliseconds (max. 30 seconds)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.sendCommand("H%d" % duration)

    async def triggerRefresh(self) -> int:
        """
        Trigger an immediate screen refresh. The combination of
        postponeRefresh and triggerRefresh can be used as an
        alternative to double-buffering to avoid flickering during display updates.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.sendCommand("H0")

    async def fade(self, brightness: int, duration: int) -> int:
        """
        Smoothly changes the brightness of the screen to produce a fade-in or fade-out
        effect.

        @param brightness : the new screen brightness
        @param duration : duration of the brightness transition, in milliseconds.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        return await self.sendCommand("+%d,%d" % (brightness, duration))

    async def newSequence(self) -> int:
        """
        Starts to record all display commands into a sequence, for later replay.
        The name used to store the sequence is specified when calling
        saveSequence(), once the recording is complete.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        self._sequence = ""
        self._recording = True
        return YAPI.SUCCESS

    async def saveSequence(self, sequenceName: str) -> int:
        """
        Stops recording display commands and saves the sequence into the specified
        file on the display internal memory. The sequence can be later replayed
        using playSequence().

        @param sequenceName : the name of the newly created sequence

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        self._recording = False
        await self._upload(sequenceName, xbytearray(self._sequence, 'latin-1'))
        # //We need to use YPRINTF("") for Objective-C
        self._sequence = ""
        return YAPI.SUCCESS

    async def playSequence(self, sequenceName: str) -> int:
        """
        Replays a display sequence previously recorded using
        newSequence() and saveSequence().

        @param sequenceName : the name of the newly created sequence

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        return await self.sendCommand("S%s" % sequenceName)

    async def pauseSequence(self, delay_ms: int) -> int:
        """
        Waits for a specified delay (in milliseconds) before playing next
        commands in current sequence. This method can be used while
        recording a display sequence, to insert a timed wait in the sequence
        (without any immediate effect). It can also be used dynamically while
        playing a pre-recorded sequence, to suspend or resume the execution of
        the sequence. To cancel a delay, call the same method with a zero delay.

        @param delay_ms : the duration to wait, in milliseconds

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        return await self.sendCommand("W%d" % delay_ms)

    async def stopSequence(self) -> int:
        """
        Stops immediately any ongoing sequence replay.
        The display is left as is.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        return await self.sendCommand("S")

    async def upload(self, pathname: str, content: xarray) -> int:
        """
        Uploads an arbitrary file (for instance a GIF file) to the display, to the
        specified full path name. If a file already exists with the same path name,
        its content is overwritten.

        @param pathname : path and name of the new file to create
        @param content : binary buffer with the content to set

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self._upload(pathname, content)

    async def copyLayerContent(self, srcLayerId: int, dstLayerId: int) -> int:
        """
        Copies the whole content of a layer to another layer. The color and transparency
        of all the pixels from the destination layer are set to match the source pixels.
        This method only affects the displayed content, but does not change any
        property of the layer object.
        Note that layer 0 has no transparency support (it is always completely opaque).

        @param srcLayerId : the identifier of the source layer (a number in range 0..layerCount-1)
        @param dstLayerId : the identifier of the destination layer (a number in range 0..layerCount-1)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        return await self.sendCommand("o%d,%d" % (srcLayerId, dstLayerId))

    async def swapLayerContent(self, layerIdA: int, layerIdB: int) -> int:
        """
        Swaps the whole content of two layers. The color and transparency of all the pixels from
        the two layers are swapped. This method only affects the displayed content, but does
        not change any property of the layer objects. In particular, the visibility of each
        layer stays unchanged. When used between one hidden layer and a visible layer,
        this method makes it possible to easily implement double-buffering.
        Note that layer 0 has no transparency support (it is always completely opaque).

        @param layerIdA : the first layer (a number in range 0..layerCount-1)
        @param layerIdB : the second layer (a number in range 0..layerCount-1)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.flushLayers()
        return await self.sendCommand("E%d,%d" % (layerIdA, layerIdB))

    async def get_displayLayer(self, layerId: int) -> Union[YDisplayLayer, None]:
        """
        Returns a YDisplayLayer object that can be used to draw on the specified
        layer. The content is displayed only when the layer is active on the
        screen (and not masked by other overlapping layers).

        @param layerId : the identifier of the layer (a number in range 0..layerCount-1)

        @return an YDisplayLayer object

        On failure, throws an exception or returns None.
        """
        layercount: int
        idx: int
        layercount = await self.get_layerCount()
        if not ((layerId >= 0) and(layerId < layercount)):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid DisplayLayer index")
            return None
        if len(self._allDisplayLayers) == 0:
            idx = 0
            while idx < layercount:
                self._allDisplayLayers.append(YDisplayLayer(self, idx))
                idx = idx + 1
        return self._allDisplayLayers[layerId]

    async def readDisplay(self, palette: list[int]) -> xarray:
        """
        Returns a color image with the current content of the display.
        The image is returned as a binary object, where each byte represents a pixel,
        from left to right and from top to bottom. The palette used to map byte
        values to RGB colors is filled into the list provided as argument.
        In all cases, the first palette entry (value 0) corresponds to the
        screen default background color.
        The image dimensions are given by the display width and height.

        @param palette : a list to be filled with the image palette

        @return a binary object if the call succeeds.

        On failure, throws an exception or returns an empty binary object.
        """
        zipmap: xarray
        zipsize: int
        zipwidth: int
        zipheight: int
        ziprotate: int
        zipcolors: int
        zipcol: int
        zipbits: int
        zipmask: int
        srcpos: int
        endrun: int
        srcpat: int
        srcbit: int
        srcval: int
        srcx: int
        srcy: int
        srci: int
        pixmap: xarray
        pixcount: int
        pixval: int
        pixpos: int
        rotmap: xarray
        pixmap = xbytearray(0)
        # Check if the display firmware has autoInvertDelay and pixels.bin support

        if await self.get_autoInvertDelay() < 0:
            # Old firmware, use uncompressed GIF output to rebuild pixmap
            zipmap = await self._download("display.gif")
            zipsize = len(zipmap)
            if zipsize == 0:
                return pixmap
            if not (zipsize >= 32):
                self._throw(YAPI.IO_ERROR, "not a GIF image")
                return pixmap
            if not ((zipmap[0] == 71) and(zipmap[2] == 70)):
                self._throw(YAPI.INVALID_ARGUMENT, "not a GIF image")
                return pixmap
            zipwidth = zipmap[6] + 256 * zipmap[7]
            zipheight = zipmap[8] + 256 * zipmap[9]
            del palette[:]
            zipcol = zipmap[13] * 65536 + zipmap[14] * 256 + zipmap[15]
            palette.append(zipcol)
            zipcol = zipmap[16] * 65536 + zipmap[17] * 256 + zipmap[18]
            palette.append(zipcol)
            pixcount = zipwidth * zipheight
            pixmap = xbytearray(pixcount)
            pixpos = 0
            srcpos = 30
            zipsize = zipsize - 2
            while srcpos < zipsize:
                # load next run size
                endrun = srcpos + 1 + zipmap[srcpos]
                srcpos = srcpos + 1
                while srcpos < endrun:
                    srcval = zipmap[srcpos]
                    srcpos = srcpos + 1
                    srcbit = 8
                    while srcbit != 0:
                        if srcbit < 3:
                            srcval = srcval + (zipmap[srcpos] << srcbit)
                            srcpos = srcpos + 1
                        pixval = (srcval & 7)
                        srcval = (srcval >> 3)
                        if not ((pixval > 1) and(pixval != 4)):
                            self._throw(YAPI.INVALID_ARGUMENT, "unexpected encoding")
                            return pixmap
                        pixmap[pixpos] = pixval
                        pixpos = pixpos + 1
                        srcbit = srcbit - 3
            return pixmap
        # New firmware, use compressed pixels.bin
        zipmap = await self._download("pixels.bin")
        zipsize = len(zipmap)
        if zipsize == 0:
            return pixmap
        if not (zipsize >= 16):
            self._throw(YAPI.IO_ERROR, "not a pixmap")
            return pixmap
        if not ((zipmap[0] == 80) and(zipmap[2] == 88)):
            self._throw(YAPI.INVALID_ARGUMENT, "not a pixmap")
            return pixmap
        zipwidth = zipmap[4] + 256 * zipmap[5]
        zipheight = zipmap[6] + 256 * zipmap[7]
        ziprotate = zipmap[8]
        zipcolors = zipmap[9]
        del palette[:]
        srcpos = 10
        srci = 0
        while srci < zipcolors:
            zipcol = zipmap[srcpos] * 65536 + zipmap[srcpos+1] * 256 + zipmap[srcpos+2]
            palette.append(zipcol)
            srcpos = srcpos + 3
            srci = srci + 1
        zipbits = 1
        while (1 << zipbits) < zipcolors:
            zipbits = zipbits + 1
        zipmask = (1 << zipbits) - 1

        pixcount = zipwidth * zipheight
        pixmap = xbytearray(pixcount)
        srcx = 0
        srcy = 0
        srcval = 0
        while srcpos < zipsize:
            # load next compression pattern byte
            srcpat = zipmap[srcpos]
            srcpos = srcpos + 1
            srcbit = 7
            while srcbit >= 0:
                # get next bitmap byte
                if (srcpat & 128) != 0:
                    srcval = zipmap[srcpos]
                    srcpos = srcpos + 1
                    if zipbits > 1:
                        srcval = (srcval << 8) + zipmap[srcpos]
                        srcpos = srcpos + 1
                srcpat = (srcpat << 1)
                pixpos = srcy * zipwidth + srcx
                # produce 8 pixels
                srci = 7 * zipbits
                while srci >= 0:
                    pixval = ((srcval >> srci) & zipmask)
                    pixmap[pixpos] = pixval
                    pixpos = pixpos + 1
                    srci = srci - zipbits
                srcy = srcy + 1
                if srcy >= zipheight:
                    srcy = 0
                    srcx = srcx + 8
                    # drop last bytes if image is not a multiple of 8
                    if srcx >= zipwidth:
                        srcbit = 0
                srcbit = srcbit - 1
        # rotate pixmap to match display orientation
        if ziprotate == 0:
            return pixmap
        if (ziprotate & 2) != 0:
            # rotate buffer 180 degrees by swapping pixels
            srcpos = 0
            pixpos = pixcount - 1
            while srcpos < pixpos:
                pixval = pixmap[srcpos]
                pixmap[srcpos] = pixmap[pixpos]
                pixmap[pixpos] = pixval
                srcpos = srcpos + 1
                pixpos = pixpos - 1
        if (ziprotate & 1) == 0:
            return pixmap
        # rotate 90 ccw: first pixel is bottom left
        rotmap = xbytearray(pixcount)
        srcx = 0
        srcy = zipwidth - 1
        srcpos = 0
        while srcpos < pixcount:
            pixval = pixmap[srcpos]
            pixpos = srcy * zipheight + srcx
            rotmap[pixpos] = pixval
            srcy = srcy - 1
            if srcy < 0:
                srcx = srcx + 1
                srcy = zipwidth - 1
            srcpos = srcpos + 1
        return rotmap

    # --- (end of generated code: YDisplay implementation)

    async def flushLayers(self) -> int:
        for it in self._allDisplayLayers:
            await it.flush_now()
        return YAPI.SUCCESS

    async def resetHiddenLayerFlags(self) -> None:
        for it in self._allDisplayLayers:
            await it.resetHiddenFlag()

    async def sendCommand(self, cmd) -> int:
        if not self._recording:
            return await self.set_command(cmd)
        self._sequence = self._sequence + cmd + '\n'
        return YAPI.SUCCESS



import sys

from yoctolib.yocto_api import YRefParam, YAPI, YModule
from yoctolib.yocto_cellular import YCellular
from yoctolib.yocto_network import YNetwork
from yoctolib.yocto_wireless import YWireless, YWlanRecord


def die(msg: str) -> None:
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# the API use local USB devices through VirtualHub
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)


def getSubnetLen(subnet: str) -> int:
    split = subnet.split('.')
    subnet_int = (16777216 * int(split[0])) \
                 + (65536 * int(split[1])) \
                 + (256 * int(split[2])) \
                 + int(split[3])
    i = 0
    while (subnet_int & (1 << i)) == 0:
        i += 1
    return 32 - i


def configureNetwork(serial: str, useDHCP: bool = True, ip: str = "",
                     subnet: str = "", gateway: str = "", dns1: str = "", dns2: str = "") -> None:
    print("Configure network function of " + serial)
    network: YNetwork = YNetwork.FindNetwork(serial + ".network")
    if not network.isOnline():
        print("Not a YoctoHub")
        return
    if useDHCP:
        print(" - Use DHCP")
        network.useDHCPauto()
    else:
        subnet_len: int = getSubnetLen(subnet)
        print(" - Use Static ip %s with subnet=%d  gateway=%s)" %
              (ip, subnet_len, gateway))
        network.useStaticIP(ip, subnet_len, gateway)
        network.set_primaryDNS(dns1)
        network.set_secondaryDNS(dns2)
    readiness: int = network.get_readiness()
    old_readiness: int = YNetwork.READINESS_INVALID
    while readiness != YNetwork.READINESS_LAN_OK and \
            readiness != YNetwork.READINESS_WWW_OK:
        YAPI.Sleep(500)
        if old_readiness != readiness:
            if readiness == YNetwork.READINESS_DOWN:
                print(" - Network is down")
            if readiness == YNetwork.READINESS_EXISTS:
                print(" - Network exists")
            if readiness == YNetwork.READINESS_LINKED:
                print(" - Network is linked")
            if readiness == YNetwork.READINESS_LAN_OK:
                print(" - Network is LAN ready")
            if readiness == YNetwork.READINESS_WWW_OK:
                print(" - Network has Internet access")
            old_readiness = readiness
        readiness = network.get_readiness()
    print("Network interface is ready and can be accessed with the IP %s." %
          (network.get_ipAddress()))
    # do not forget to call saveToFlash() to make these settings persistent


def configureWireless(serial: str, ssid: str, passkey: str = "", useDHCP: bool = True, ip: str = "", subnet: str = "",
                      gateway: str = "", dns1: str = "", dns2: str = "") -> None:
    print("Configure Wireless for " + serial)
    wireless: YWireless = YWireless.FindWireless(serial + ".wireless")
    network: YNetwork = YNetwork.FindNetwork(serial + ".network")
    if not wireless.isOnline() or not network.isOnline():
        print("Not a wireless YoctoHub")
        return
    if wireless.get_wlanState() == YWireless.WLANSTATE_INVALID:
        print("YoctoHub " + serial + " is too old. ")
        print("please the firmware to use this function")
        return

    # set wireless settings
    wireless.joinNetwork(ssid, passkey)
    # set IP settings
    if useDHCP:
        print(" - Use DHCP")
        network.useDHCPauto()
    else:
        subnet_len = getSubnetLen(subnet)
        print(" - Use Static ip %s with  subnet=%d and gateway =%s)" %
              (ip, subnet_len, gateway))
        network.useStaticIP(ip, subnet_len, gateway)
        network.set_primaryDNS(dns1)
        network.set_secondaryDNS(dns2)
    # ensure that the wireless settings are correct
    last_message: str = ""
    networkState: int = wireless.get_wlanState()
    while networkState != YWireless.WLANSTATE_CONNECTED and \
            networkState != YWireless.WLANSTATE_REJECTED:
        message: str = wireless.get_message()
        if last_message != message:
            print(" - " + message)
            last_message = message
        YAPI.Sleep(500)
        networkState = wireless.get_wlanState()
    if networkState == YWireless.WLANSTATE_REJECTED:
        print("Unable to connect to %s network : %s" %
              (ssid, wireless.get_message()))
        return
    readiness: int = network.get_readiness()
    old_readiness: int = YNetwork.READINESS_INVALID
    while readiness != YNetwork.READINESS_LAN_OK and \
            readiness != YNetwork.READINESS_WWW_OK:
        if old_readiness != readiness:
            if readiness == YNetwork.READINESS_DOWN:
                print(" - Network is down")
            if readiness == YNetwork.READINESS_EXISTS:
                print(" - Network exists")
            if readiness == YNetwork.READINESS_LINKED:
                print(" - Network is linked")
            if readiness == YNetwork.READINESS_LAN_OK:
                print(" - Network is LAN ready OK")
            if readiness == YNetwork.READINESS_WWW_OK:
                print(" - Network as WWW ready")
            old_readiness = readiness
        YAPI.Sleep(500)
        readiness = network.get_readiness()

    print("Wireless interface ready and can be accessed with the IP %s (link=%d%%)." %
          (network.get_ipAddress(), wireless.get_linkQuality()))
    # do not forget to call saveToFlash() to make these settings persistent


def getAvailableWirelessNetwork(serial: str) -> None:
    print("Scan available wireless network from " + serial)
    wireless: YWireless = YWireless.FindWireless(serial + ".wireless")
    if not wireless.isOnline():
        print("Not a wireless YoctoHub")
        return
    if wireless.get_wlanState() == YWireless.WLANSTATE_INVALID:
        print("YoctoHub " + serial + " is too old. ")
        print("please the firmware to use this function")
        return
    wireless.startWlanScan()
    networkState: int = wireless.get_wlanState()
    last_message: str = ""
    while networkState == YWireless.WLANSTATE_DOWN or \
            networkState == YWireless.WLANSTATE_SCANNING:
        message: str = wireless.get_message()
        if last_message != message:
            print(" - %s" % message)
            last_message = message
        YAPI.Sleep(100)
        networkState = wireless.get_wlanState()
    wlans: list[YWlanRecord] = wireless.get_detectedWlans()
    print("Detected networks:")
    for wl in wlans:
        assert isinstance(wl, YWlanRecord)
        print(" - ssid:%s channel:%d quality:%d security:%s" %
              (wl.get_ssid(), wl.get_channel(), wl.get_linkQuality(), wl.get_security()))


def configureCelluar(serial: str,
                     pin_number: str = "",
                     operator: str = "",
                     apn_host: str = "",
                     apn_user: str = "",
                     apn_pass: str = "",
                     data_mode: int = YCellular.ENABLEDATA_HOMENETWORK) -> None:
    print("Configure Cellular for " + serial)
    cellular: YCellular = YCellular.FindCellular(serial + ".cellular")
    network: YNetwork = YNetwork.FindNetwork(serial + ".network")

    cellular.set_pin(pin_number)
    cellular.set_lockedOperator(operator)
    cellular.set_apn(apn_host)
    cellular.set_apnAuth(apn_user, apn_pass)
    cellular.set_enableData(data_mode)
    readiness: int = network.get_readiness()
    old_msg: str = ""
    msg: str = ""
    while readiness != YNetwork.READINESS_WWW_OK:
        if readiness == YNetwork.READINESS_DOWN:
            msg = " - Network is down (%s)" % cellular.get_message()
        elif readiness == YNetwork.READINESS_EXISTS:
            msg = " - Network exists (%s)" % (cellular.get_message())
        elif readiness == YNetwork.READINESS_LINKED:
            msg = " - Network is linked (%s:%s)" % \
                  (cellular.get_cellOperator(), cellular.get_message())
        elif readiness == YNetwork.READINESS_LAN_OK:
            msg = " - Network is LAN ready (%s:%s)" % \
                  (cellular.get_cellOperator(), cellular.get_message())
        elif readiness == YNetwork.READINESS_WWW_OK:
            msg = " - Network has Internet access (%s:%s)" % \
                  (cellular.get_cellOperator(), cellular.get_message())
        if msg != old_msg:
            print(msg)
            old_msg = msg
        YAPI.Sleep(100)
        readiness = network.get_readiness()
    print("Network interface is ready and can be accessed with the IP %s." %
          (network.get_ipAddress()))


m: YModule = YModule.FirstModule()
while m is not None:
    serial_number: str = m.get_serialNumber()
    print()
    print(serial_number + ' (' + m.get_productName() + ')')
    if m.hasFunction("wireless"):
        # getAvailableWirelessNetwork(serial_number)
        configureWireless(serial_number, "SWEETSHORTWPA")
    elif m.hasFunction("cellular"):
        configureCelluar(serial_number)
    elif m.hasFunction("network"):
        configureNetwork(serial_number)
    # do not forget to call saveToFlash() to make these settings persistent
    m.saveToFlash()
    m = m.nextModule()

YAPI.FreeAPI()



import math
import sys

from yoctolib.yocto_api import YAPI, YRefParam
from yoctolib.yocto_display import YDisplay, YDisplayLayer


def die(msg: str) -> None:
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# this is the recusive function to draw 1/3nd of the Von Koch flake
def recursiveLine(layer: YDisplayLayer, x0: float, y0: float, x1: float, y1: float, deep: int):
    if deep <= 0:
        layer.moveTo(int(x0 + 0.5), int(y0 + 0.5))
        layer.lineTo(int(x1 + 0.5), int(y1 + 0.5))
    else:
        dx = (x1 - x0) / 3
        dy = (y1 - y0) / 3
        mx = ((x0 + x1) / 2) + (0.87 * (y1 - y0) / 3)
        my = ((y0 + y1) / 2) - (0.87 * (x1 - x0) / 3)
        recursiveLine(layer, x0, y0, x0 + dx, y0 + dy, deep - 1)
        recursiveLine(layer, x0 + dx, y0 + dy, mx, my, deep - 1)
        recursiveLine(layer, mx, my, x1 - dx, y1 - dy, deep - 1)
        recursiveLine(layer, x1 - dx, y1 - dy, x1, y1, deep - 1)


# setup the API to use local USB devices
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)

# In order to use a specific device, invoke the script as
#   python doubleBuffering.py [serial_number]
# or
#   python doubleBuffering.py [logical_name]
target: str = 'any'
if len(sys.argv) > 1:
    target = sys.argv[1]

if target == 'any':
    # retrieve any compatible module
    disp: YDisplay = YDisplay.FirstDisplay()
    if disp is None:
        die('No display module connected')
    target = disp.get_serialNumber()

# retrieve specified module
disp = YDisplay.FindDisplay(target + ".display")
if not disp.isOnline():
    die("Display '%s' not connected" % target)

# display clean up
disp.resetAll()

l1: YDisplayLayer = disp.get_displayLayer(1)
l2: YDisplayLayer = disp.get_displayLayer(2)
l1.hide()  # L1 is hidden, l2 stays visible
centerX: int = disp.get_displayWidth() // 2
centerY: int = disp.get_displayHeight() // 2
radius: float = disp.get_displayHeight() / 2
a: int = 0

while True:
    # we draw in the hidden layer
    l1.clear()
    for i in range(0, 3):
        recursiveLine(l1, centerX + radius * math.cos(a + i * 2.094),
                      centerY + radius * math.sin(a + i * 2.094),
                      centerX + radius * math.cos(a + (i + 1) * 2.094),
                      centerY + radius * math.sin(a + (i + 1) * 2.094), 2)
    # then we swap contents with the visible layer

    disp.swapLayerContent(1, 2)
    # change the flake angle
    a += 0.1257



# ********************************************************************
#
#  $Id: helloworld.py 66453 2025-05-09 10:25:49Z seb $
#
#  An example that show how to use a  Yocto-Knob
#
#  You can find more information on our web site:
#   Yocto-Watt documentation:
#      https://www.yoctopuce.com/EN/products/yocto-watt/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-typedpython-EN.html
#
# *********************************************************************
import sys

from yoctolib.yocto_api import YAPI, YRefParam
from yoctolib.yocto_power import YPower


def die(msg):
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# the API use local USB devices through VirtualHub
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)

# To use a specific device, invoke the script as
#   python helloworld.py [serial_number]
# or
#   python helloworld.py [logical_name]
target: str = 'any'
if len(sys.argv) > 1:
    target = sys.argv[1]

if target == 'any':
    # retreive any Power sensor
    func: YPower = YPower.FirstPower()
    if func is None:
        die('No Yocto-Watt connected')
    target = func.get_serialNumber()

sensor: YPower = YPower.FindPower(target + '.power')

if not sensor.isOnline():
    die('device not connected')
while sensor.isOnline():
    print("Power :  " + "%2.1f" % sensor.get_currentValue() + "W (Ctrl-C to stop)")
    YAPI.Sleep(1000)
YAPI.FreeAPI()



# ********************************************************************
#
#  $Id: helloworld.py 66453 2025-05-09 10:25:49Z seb $
#
#  An example that shows how to use a  Yocto-LatchedRelay
#
#  You can find more information on our web site:
#   Yocto-LatchedRelay documentation:
#      https://www.yoctopuce.com/EN/products/yocto-latchedrelay/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-typedpython-EN.html
#
# *********************************************************************
import sys

from yoctolib.yocto_api import YRefParam, YAPI
from yoctolib.yocto_relay import YRelay


def die(msg):
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# the API use local USB devices through VirtualHub
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)

# To use a specific device, invoke the script as
#   python helloworld.py [serial_number]
# or
#   python helloworld.py [logical_name]
target: str = 'any'
if len(sys.argv) > 1:
    target = sys.argv[1]

if target == 'any':
    # retrieve any Relay
    sensor: YRelay = YRelay.FirstRelay()
    if sensor is None:
        die('No Yocto-LatchedRelay connected')
    target: str = sensor.get_serialNumber()

# retrieve specified functions
relay: YRelay = YRelay.FindRelay(target + ".relay1")
if not (relay.isOnline()):
    die('device not connected')

print("Use device %s" % relay.get_serialNumber())

relay.set_state(YRelay.STATE_B)
YAPI.Sleep(500)
relay.set_state(YRelay.STATE_A)
YAPI.FreeAPI()



import sys

from yoctolib.yocto_api import YRefParam, YAPI, YModule, YFirmwareUpdate


def die(msg):
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


def upgradeSerialList(all_serials: list[str]):
    for serial in all_serials:
        module: YModule = YModule.FindModule(serial)
        product: str = module.get_productName()
        serial: str = module.get_serialNumber()
        current: str = module.get_firmwareRelease()
        # check if new firmware is available on yoctopuce.com
        new_firmware: str = module.checkFirmware("www.yoctopuce.com", True)
        if new_firmware == "":
            print(product + " " + serial + "(rev=" + current + ") is up to date")
        else:
            print(product + " " + serial + "(rev=" + current + ") need be updated with firmware : ")
            print("    " + new_firmware)
            # execute the firmware upgrade
            update: YFirmwareUpdate = module.updateFirmware(new_firmware)
            status: int = update.startUpdate()
            while 100 > status >= 0:
                new_status: int = update.get_progress()
                if new_status != status:
                    print(str(new_status) + "% " + update.get_progressMessage())
                YAPI.Sleep(500, errmsg)
                status = new_status
            if status < 0:
                die("Firmware Update failed: " + update.get_progressMessage())
            else:
                if module.isOnline():
                    print(str(status) + "% Firmware Updated Successfully!")
                else:
                    die(str(status) + " Firmware Update failed: module " + serial + "is not online")


# the API use local USB devices through VirtualHub
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    die("RegisterHub failed: " + errmsg.value)

i: int = 1
for i in range(1, len(sys.argv)):
    print("Update module connected to hub " + sys.argv[i])
    # Setup the API to use local USB devices
    if YAPI.RegisterHub(sys.argv[i], errmsg) != YAPI.SUCCESS:
        die("init error" + errmsg.value)

hubs: list[str] = []
shield: list[str] = []
devices: list[str] = []
# fist step construct the list of all hub /shield and devices connected
module: YModule = YModule.FirstModule()
while module is not None:
    product: str = module.get_productName()
    serial: str = module.get_serialNumber()
    if product == "YoctoHub-Shield":
        shield.append(serial)
    elif product[0:8] == "YoctoHub":
        hubs.append(serial)
    elif product[0:9] != "VirtualHub":
        devices.append(serial)
    module = module.nextModule()
# fist upgrades all Hubs...
upgradeSerialList(hubs)
# ... then all shield..
upgradeSerialList(shield)
# ... and finally all devices
upgradeSerialList(devices)
print("All devices are now up to date")
YAPI.FreeAPI()



# ********************************************************************
#
#  $Id: helloworld.py 66453 2025-05-09 10:25:49Z seb $
#
#  An example that shows how to use a  Yocto-Display
#
#  You can find more information on our web site:
#   Yocto-Display documentation:
#      https://www.yoctopuce.com/EN/products/yocto-display/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-typedpython-EN.html
#
# *********************************************************************
import sys

from yoctolib.yocto_api import YRefParam, YAPI
from yoctolib.yocto_display import YDisplay, YDisplayLayer


def die(msg: str) -> None:
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# the API use local USB devices through VirtualHub
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)

# To use a specific device, invoke the script as
#   python helloworld.py [serial_number]
# or
#   python helloworld.py [logical_name]
target: str = 'any'
if len(sys.argv) > 1:
    target = sys.argv[1]

if target == 'any':
    # retrieve any display
    tmp: YDisplay = YDisplay.FirstDisplay()
    if tmp is None:
        die('No module connected')
    target = tmp.get_serialNumber()

# retrieve specified functions
disp = YDisplay.FindDisplay(target + ".display")
if not disp.isOnline():
    die("Yocto-Display '%s' not connected" % target)

print("Use device %s" % disp.get_serialNumber())

# display clean up
disp.resetAll()

# retrieve the display size
w: int = disp.get_displayWidth()
h: int = disp.get_displayHeight()

# retrieve the first layer
l0: YDisplayLayer = disp.get_displayLayer(0)
l0.clear()

# display a text in the middle of the screen
l0.drawText(w // 2, h // 2, YDisplayLayer.ALIGN.CENTER, "Hello world!")

# visualize each corner
l0.moveTo(0, 5)
l0.lineTo(0, 0)
l0.lineTo(5, 0)
l0.moveTo(0, h - 6)
l0.lineTo(0, h - 1)
l0.lineTo(5, h - 1)
l0.moveTo(w - 1, h - 6)
l0.lineTo(w - 1, h - 1)
l0.lineTo(w - 6, h - 1)
l0.moveTo(w - 1, 5)
l0.lineTo(w - 1, 0)
l0.lineTo(w - 6, 0)

# draw a circle in the top left corner of layer 1
l1: YDisplayLayer = disp.get_displayLayer(1)
l1.clear()
l1.drawCircle(h // 8, h // 8, h // 8)

# and animate the layer
print("Use Ctrl-C to stop")
x: int = 0
y: int = 0
vx: int = 1
vy: int = 1
while disp.isOnline():
    x += vx
    y += vy
    if x < 0 or x > w - (h / 4):
        vx = -vx
    if y < 0 or y > h - (h / 4):
        vy = -vy
    l1.setLayerPosition(x, y, 0)
    YAPI.Sleep(5, errmsg)
YAPI.FreeAPI()



import asyncio
import logging
import sys

from yoctolib.yocto_api_aio import YRefParam, YAPI, YModule, YFirmwareUpdate


async def die(msg):
    await YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


async def upgradeSerialList(all_serials: list[str]):
    for serial in all_serials:
        module: YModule = YModule.FindModule(serial)
        product: str = await module.get_productName()
        serial: str = await module.get_serialNumber()
        current: str = await module.get_firmwareRelease()
        # check if new firmware is available on yoctopuce.com
        new_firmware: str = await module.checkFirmware("www.yoctopuce.com", True)
        if new_firmware == "":
            print(product + " " + serial + "(rev=" + current + ") is up to date")
        else:
            print(product + " " + serial + "(rev=" + current + ") need be updated with firmware : ")
            print("    " + new_firmware)
            # execute the firmware upgrade
            update: YFirmwareUpdate = await module.updateFirmware(new_firmware)
            status: int = await update.startUpdate()
            while 100 > status >= 0:
                new_status: int = await update.get_progress()
                if new_status != status:
                    print(str(new_status) + "% " + await update.get_progressMessage())
                await YAPI.Sleep(500)
                status = new_status
            if status < 0:
                await die("Firmware Update failed: " + await update.get_progressMessage())
            else:
                if await module.isOnline():
                    print(str(status) + "% Firmware Updated Successfully!")
                else:
                    await die(str(status) + " Firmware Update failed: module " + serial + "is not online")

async def main():

    # the API use local USB devices through VirtualHub
    errmsg: YRefParam = YRefParam()
    if await YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
        await die("RegisterHub failed: " + errmsg.value)

    i: int = 1
    for i in range(1, len(sys.argv)):
        print("Update module connected to hub " + sys.argv[i])
        # Setup the API to use local USB devices
        if await YAPI.RegisterHub(sys.argv[i], errmsg) != YAPI.SUCCESS:
            await die("init error" + errmsg.value)

    hubs: list[str] = []
    shield: list[str] = []
    devices: list[str] = []
    # fist step construct the list of all hub /shield and devices connected
    module: YModule = YModule.FirstModule()
    while module is not None:
        product: str = await module.get_productName()
        serial: str = await module.get_serialNumber()
        if product == "YoctoHub-Shield":
            shield.append(serial)
        elif product[0:8] == "YoctoHub":
            hubs.append(serial)
        elif product[0:9] != "VirtualHub":
            devices.append(serial)
        module = module.nextModule()
    # fist upgrades all Hubs...
    await upgradeSerialList(hubs)
    # ... then all shield..
    await upgradeSerialList(shield)
    # ... and finally all devices
    await upgradeSerialList(devices)
    print("All devices are now up to date")
    await YAPI.FreeAPI()


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    asyncio.run(main(), debug=True)



import sys
from typing import Union

from yoctolib.yocto_api import YRefParam, YAPI, YModule
from yoctolib.yocto_hubport import YHubPort


def die(msg: str) -> None:
    YAPI.FreeAPI()
    sys.exit(msg)


class YoctoShield(object):
    _serial: str
    _subDevices: list[str]

    def __init__(self, serial: str) -> None:
        self._serial = serial
        self._subDevices = []

    def getSerial(self) -> str:
        return self._serial

    def addSubdevice(self, serial: str) -> bool:
        for i in range(1, 5):
            p: YHubPort = YHubPort.FindHubPort("%s.hubPort%d" % (self._serial, i))
            if p.get_logicalName() == serial:
                self._subDevices.append(serial)
                return True
        return False

    def removeSubDevice(self, serial: str) -> None:
        if serial in self._subDevices:
            self._subDevices.remove(serial)

    def describe(self) -> None:
        print("  " + self._serial)
        for subdevice in self._subDevices:
            print("    " + subdevice)


class RootDevice(object):
    _serial: str
    _url: str
    _shields: list[YoctoShield]
    _subDevices: list[str]

    def __init__(self, serial: str, url: str) -> None:
        self._serial = serial
        self._url = url
        self._shields = []
        self._subDevices = []

    def getSerial(self) -> str:
        return self._serial

    def addSubDevice(self, serial: str) -> None:
        if serial[:7] == "YHUBSHL":
            self._shields.append(YoctoShield(serial))
        else:
            # Device to plug look if the device is plugged on a shield
            for shield in self._shields:
                if shield.addSubdevice(serial):
                    return
            self._subDevices.append(serial)

    def removeSubDevice(self, serial: str) -> None:
        if serial in self._subDevices:
            self._subDevices.remove(serial)
        for yoctoShield in reversed(list(self._shields)):
            if yoctoShield.getSerial() == serial:
                self._shields.remove(yoctoShield)
                break
            else:
                yoctoShield.removeSubDevice(serial)

    def describe(self) -> None:
        print(self._serial + " (" + self._url + ")")
        for subdevice in self._subDevices:
            print("  " + subdevice)
        for shield in self._shields:
            shield.describe()


__rootDevices: list[RootDevice] = []


def getYoctoHub(serial: str) -> Union[RootDevice, None]:
    for rootDevice in __rootDevices:
        if rootDevice.getSerial() == serial:
            return rootDevice
    return None


def addRootDevice(serial: str, url: str) -> RootDevice:
    for rootDevice in __rootDevices:
        if rootDevice.getSerial() == serial:
            return rootDevice
    rootDevice = RootDevice(serial, url)
    __rootDevices.append(rootDevice)
    return rootDevice


def showNetwork() -> None:
    print("**** device inventory *****")
    for hub in __rootDevices:
        hub.describe()


def deviceArrival(module: YModule) -> None:
    serial: str = module.get_serialNumber()
    parentHub: str = module.get_parentHub()
    if parentHub == "":
        # root device
        url: str = module.get_url()
        addRootDevice(serial, url)
    else:
        hub: Union[RootDevice, None] = getYoctoHub(parentHub)
        if hub is not None:
            hub.addSubDevice(serial)


def deviceRemoval(module: YModule) -> None:
    serial: str = module.get_serialNumber()
    for rootDevice in reversed(list(__rootDevices)):
        rootDevice.removeSubDevice(serial)
        if rootDevice.getSerial() == serial:
            __rootDevices.remove(rootDevice)


# the API use local USB devices through VirtualHub
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    die("RegisterHub failed: " + errmsg.value)

if YAPI.RegisterHub("net", errmsg) != YAPI.SUCCESS:
    die("RegisterHub error: " + errmsg.value)

YAPI.RegisterDeviceArrivalCallback(deviceArrival)
YAPI.RegisterDeviceRemovalCallback(deviceRemoval)

print("Waiting for hubs to signal themselves...")
# wait for 5 seconds, doing nothing.
# noinspection InfiniteLoopStatement
while True:
    YAPI.UpdateDeviceList(errmsg)  # traps plug/unplug events
    YAPI.Sleep(1000, errmsg)  # traps others events
    showNetwork()



import sys

from yoctolib.yocto_api import YRefParam, YAPI
from yoctolib.yocto_colorledcluster import YColorLedCluster


def die(msg: str) -> None:
    YAPI.FreeAPI()
    sys.exit(msg)


def main():
    # the API use local USB devices through VirtualHub
    errmsg: YRefParam = YRefParam()
    if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
        sys.exit("RegisterHub failed: " + errmsg.value)

    leds: YColorLedCluster = YColorLedCluster.FirstColorLedCluster()
    if leds is None:
        sys.exit("No Color Led cluster found (check USB cable)")

    leds.resetBlinkSeq(0)  # cleans the sequence 0
    leds.addRgbMoveToBlinkSeq(0, 0x400000, 0)  # move instantaneously to red 25%
    leds.addRgbMoveToBlinkSeq(0, 0x400000, 500)  # stay on the same color for 0.5sec
    leds.addRgbMoveToBlinkSeq(0, 0x000040, 0)  # move instantaneously to red 25%
    leds.addRgbMoveToBlinkSeq(0, 0x000040, 500)  # stay on the same color for 0.5sec
    leds.linkLedToBlinkSeq(0, 1, 0, 0)  # link led 0 to sequence 0
    leds.linkLedToBlinkSeqAtPowerOn(0, 1, 0, 0)  # led 0 will automatically be linked to sequence 0  at startup
    leds.set_blinkSeqStateAtPowerOn(0, 1)  # sequence 0 will automatically run at startup
    leds.saveBlinkSeq(0)  # save the sequence on flash memory

    leds.resetBlinkSeq(1)  # cleans the sequence  1
    leds.addHslMoveToBlinkSeq(1, 0x00FF20, 2000)  # Circle over all hue values
    leds.addHslMoveToBlinkSeq(1, 0x55FF20, 2000)  # Saturation =100%
    leds.addHslMoveToBlinkSeq(1, 0xAAFF20, 2000)  # luminosity = 12%
    leds.linkLedToBlinkSeq(1, 1, 1, 0)  # link led 1 to sequence 1
    leds.linkLedToBlinkSeqAtPowerOn(1, 1, 1, 0)  # led 1 will automatically be linked to sequence 1  at startup
    leds.set_blinkSeqStateAtPowerOn(1, 1)  # sequence 1  will automatically run at startup
    leds.saveBlinkSeq(1)  # save the sequence on flash memory

    leds.saveLedsConfigAtPowerOn()  # All leds configuration

    leds.startBlinkSeq(0)  # start sequence 0
    leds.startBlinkSeq(1)  # start sequence 1

    print("The leds are now blinking autonomously")
    YAPI.FreeAPI()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YVirtualSensor
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YVirtualSensor
version: PATCH_WITH_VERSION
requires: yocto_virtualsensor_aio
requires: yocto_api
provides: YVirtualSensor
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    const = lambda obj: obj
    _IS_MICROPYTHON = False
    _DYNAMIC_HELPERS = False
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_virtualsensor_aio import YVirtualSensor as YVirtualSensor_aio
from .yocto_api import (
    YAPIContext, YAPI, YAPI_aio, YSensor, YMeasure
)

# --- (YVirtualSensor class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YVirtualSensorValueCallback = Union[Callable[['YVirtualSensor', str], Any], None]
        YVirtualSensorTimedReportCallback = Union[Callable[['YVirtualSensor', YMeasure], Any], None]
    except TypeError:
        YVirtualSensorValueCallback = Union[Callable, Awaitable]
        YVirtualSensorTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YVirtualSensor(YSensor):
    """
    The YVirtualSensor class allows you to use Yoctopuce virtual sensors.
    These sensors make it possible to show external data collected by the user
    as a Yoctopuce Sensor. This class inherits from YSensor class the core
    functions to read measurements, to register callback functions, and to access
    the autonomous datalogger. It adds the ability to change the sensor value as
    needed, or to mark current value as invalid.

    """
    _aio: YVirtualSensor_aio
    # --- (end of YVirtualSensor class start)
    if not _IS_MICROPYTHON:
        # --- (YVirtualSensor return codes)
        INVALIDVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        # --- (end of YVirtualSensor return codes)


    # --- (YVirtualSensor implementation)

    @classmethod
    def FindVirtualSensor(cls, func: str) -> YVirtualSensor:
        """
        Retrieves a virtual sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the virtual sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YVirtualSensor.isOnline() to test if the virtual sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a virtual sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the virtual sensor, for instance
                MyDevice.virtualSensor1.

        @return a YVirtualSensor object allowing you to drive the virtual sensor.
        """
        return cls._proxy(cls, YVirtualSensor_aio.FindVirtualSensorInContext(YAPI_aio, func))

    @classmethod
    def FindVirtualSensorInContext(cls, yctx: YAPIContext, func: str) -> YVirtualSensor:
        """
        Retrieves a virtual sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the virtual sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YVirtualSensor.isOnline() to test if the virtual sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a virtual sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the virtual sensor, for instance
                MyDevice.virtualSensor1.

        @return a YVirtualSensor object allowing you to drive the virtual sensor.
        """
        return cls._proxy(cls, YVirtualSensor_aio.FindVirtualSensorInContext(yctx._aio, func))

    @classmethod
    def FirstVirtualSensor(cls) -> Union[YVirtualSensor, None]:
        """
        Starts the enumeration of virtual sensors currently accessible.
        Use the method YVirtualSensor.nextVirtualSensor() to iterate on
        next virtual sensors.

        @return a pointer to a YVirtualSensor object, corresponding to
                the first virtual sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YVirtualSensor_aio.FirstVirtualSensorInContext(YAPI_aio))

    @classmethod
    def FirstVirtualSensorInContext(cls, yctx: YAPIContext) -> Union[YVirtualSensor, None]:
        """
        Starts the enumeration of virtual sensors currently accessible.
        Use the method YVirtualSensor.nextVirtualSensor() to iterate on
        next virtual sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YVirtualSensor object, corresponding to
                the first virtual sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YVirtualSensor_aio.FirstVirtualSensorInContext(yctx._aio))

    def nextVirtualSensor(self) -> Union[YVirtualSensor, None]:
        """
        Continues the enumeration of virtual sensors started using yFirstVirtualSensor().
        Caution: You can't make any assumption about the returned virtual sensors order.
        If you want to find a specific a virtual sensor, use VirtualSensor.findVirtualSensor()
        and a hardwareID or a logical name.

        @return a pointer to a YVirtualSensor object, corresponding to
                a virtual sensor currently online, or a None pointer
                if there are no more virtual sensors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextVirtualSensor())

    if not _DYNAMIC_HELPERS:
        def set_unit(self, newval: str) -> int:
            """
            Changes the measuring unit for the measured value.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the measuring unit for the measured value

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_unit(newval))

    if not _DYNAMIC_HELPERS:
        def set_currentRawValue(self, newval: float) -> int:
            """
            Changes the current value of the sensor (raw value, before calibration).

            @param newval : a floating point number corresponding to the current value of the sensor (raw
            value, before calibration)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_currentRawValue(newval))

    if not _DYNAMIC_HELPERS:
        def set_sensorState(self, newval: int) -> int:
            return self._run(self._aio.set_sensorState(newval))

    if not _DYNAMIC_HELPERS:
        def set_invalidValue(self, newval: float) -> int:
            """
            Changes the invalid value of the sensor, returned if the sensor is read when in invalid state
            (for instance before having been set). Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a floating point number corresponding to the invalid value of the sensor, returned
            if the sensor is read when in invalid state
                    (for instance before having been set)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_invalidValue(newval))

    if not _DYNAMIC_HELPERS:
        def get_invalidValue(self) -> float:
            """
            Returns the invalid value of the sensor, returned if the sensor is read when in invalid state
            (for instance before having been set).

            @return a floating point number corresponding to the invalid value of the sensor, returned if the
            sensor is read when in invalid state
                    (for instance before having been set)

            On failure, throws an exception or returns YVirtualSensor.INVALIDVALUE_INVALID.
            """
            return self._run(self._aio.get_invalidValue())

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YVirtualSensorValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is called once when it is registered, passing the current advertised value
            of the function, provided that it is not an empty string.
            The callback is then invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YVirtualSensorTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def set_sensorAsInvalid(self) -> int:
            """
            Changes the current sensor state to invalid (as if no value would have been ever set).

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_sensorAsInvalid())

    # --- (end of YVirtualSensor implementation)




# ********************************************************************
#
#  $Id: helloworld.py 66453 2025-05-09 10:25:49Z seb $
#
#  An example that shows how to use a  Yocto-Color
#
#  You can find more information on our web site:
#   Yocto-Color documentation:
#      https://www.yoctopuce.com/EN/products/yocto-color/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-typedpython-EN.html
#
# *********************************************************************
import sys

from yoctolib.yocto_api import YAPI, YRefParam
from yoctolib.yocto_colorled import YColorLed


def die(msg):
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# the API use local USB devices through VirtualHub
errmsg: YRefParam = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)

# To use a specific device, invoke the script as
#   python helloworld.py [serial_number]
# or
#   python helloworld.py [logical_name]
target: str = 'any'
if len(sys.argv) > 1:
    target = sys.argv[1]

if target == 'any':
    # retreive any RGB led
    func: YColorLed = YColorLed.FirstColorLed()
    if func is None:
        die('No Yocto-Color connected')
    target = func.get_serialNumber()

led1: YColorLed = YColorLed.FindColorLed(target + '.colorLed1')
led2: YColorLed = YColorLed.FindColorLed(target + '.colorLed2')

all_colors = [0xff0000, 0x00ff00, 0x0000ff]
for color in all_colors:
    print("Change color to 0x%06x" % color)
    led1.set_rgbColor(color)  # immediate transition
    led2.rgbMove(color, 1000)  # smooth transition
    YAPI.Sleep(1000)

YAPI.FreeAPI()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_inputcapture_aio.py 71691 2026-02-02 06:59:29Z mvuilleu $
#
#  Implements the asyncio YInputCapture API for InputCapture functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YInputCapture
version: PATCH_WITH_VERSION
requires: yocto_api_aio
provides: YInputCapture YInputCaptureData
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    const = lambda obj: obj
    _IS_MICROPYTHON = False
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True  # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, YFunction, HwId, hwid2str,
    xarray, xbytearray, xStringIO
)

# --- (generated code: YInputCaptureData class start)
# noinspection PyProtectedMember
class YInputCaptureData:
    """
    InputCaptureData objects represent raw data
    sampled by the analog/digital converter present in
    a Yoctopuce electrical sensor. When several inputs
    are samples simultaneously, their data are provided
    as distinct series.

    """
    # --- (end of generated code: YInputCaptureData class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YInputCaptureData return codes)
        pass
        # --- (end of generated code: YInputCaptureData return codes)
        pass

    # --- (generated code: YInputCaptureData attributes declaration)
    _fmt: int
    _var1size: int
    _var2size: int
    _var3size: int
    _nVars: int
    _recOfs: int
    _nRecs: int
    _samplesPerSec: int
    _trigType: int
    _trigVal: float
    _trigPos: int
    _trigUTC: float
    _var1unit: str
    _var2unit: str
    _var3unit: str
    _var1samples: list[float]
    _var2samples: list[float]
    _var3samples: list[float]
    # --- (end of generated code: YInputCaptureData attributes declaration)

    def __init__(self, yfun: YFunction, json_data: xarray):
        # --- (generated code: YInputCaptureData constructor)
        self._fmt = 0
        self._var1size = 0
        self._var2size = 0
        self._var3size = 0
        self._nVars = 0
        self._recOfs = 0
        self._nRecs = 0
        self._samplesPerSec = 0
        self._trigType = 0
        self._trigVal = 0.0
        self._trigPos = 0
        self._trigUTC = 0.0
        self._var1unit = ''
        self._var2unit = ''
        self._var3unit = ''
        self._var1samples = []
        self._var2samples = []
        self._var3samples = []
        # --- (end of generated code: YInputCaptureData constructor)
        self._decodeSnapBin(json_data)

    @staticmethod
    def _throw(errType: int, errMsg: str):
        if not YAPI._ExceptionsDisabled:
            raise YAPI_Exception(errType, errMsg)

    # --- (generated code: YInputCaptureData implementation)
    def _decodeU16(self, sdata: xarray, ofs: int) -> int:
        v: int
        v = sdata[ofs]
        v = v + 256 * sdata[ofs+1]
        return v

    def _decodeU32(self, sdata: xarray, ofs: int) -> float:
        v: float
        v = self._decodeU16(sdata, ofs)
        v = v + 65536.0 * self._decodeU16(sdata, ofs+2)
        return v

    def _decodeVal(self, sdata: xarray, ofs: int, len: int) -> float:
        v: float
        b: float
        v = self._decodeU16(sdata, ofs)
        b = 65536.0
        ofs = ofs + 2
        len = len - 2
        while len > 0:
            v = v + b * sdata[ofs]
            b = b * 256
            ofs = ofs + 1
            len = len - 1
        if v > (b/2):
            # negative number
            v = v - b
        return v

    def _decodeSnapBin(self, sdata: xarray) -> int:
        buffSize: int
        recOfs: int
        ms: int
        recSize: int
        count: int
        mult1: int
        mult2: int
        mult3: int
        v: float

        buffSize = len(sdata)
        if not (buffSize >= 24):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid snapshot data (too short)")
            return YAPI.INVALID_ARGUMENT
        self._fmt = sdata[0]
        self._var1size = sdata[1] - 48
        self._var2size = sdata[2] - 48
        self._var3size = sdata[3] - 48
        if not (self._fmt == 83):
            self._throw(YAPI.INVALID_ARGUMENT, "Unsupported snapshot format")
            return YAPI.INVALID_ARGUMENT
        if not ((self._var1size >= 2) and(self._var1size <= 4)):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid sample size")
            return YAPI.INVALID_ARGUMENT
        if not ((self._var2size >= 0) and(self._var1size <= 4)):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid sample size")
            return YAPI.INVALID_ARGUMENT
        if not ((self._var3size >= 0) and(self._var1size <= 4)):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid sample size")
            return YAPI.INVALID_ARGUMENT
        if self._var2size == 0:
            self._nVars = 1
        else:
            if self._var3size == 0:
                self._nVars = 2
            else:
                self._nVars = 3
        recSize = self._var1size + self._var2size + self._var3size
        self._recOfs = self._decodeU16(sdata, 4)
        self._nRecs = self._decodeU16(sdata, 6)
        self._samplesPerSec = self._decodeU16(sdata, 8)
        self._trigType = self._decodeU16(sdata, 10)
        self._trigVal = self._decodeVal(sdata, 12, 4) / 1000
        self._trigPos = self._decodeU16(sdata, 16)
        ms = self._decodeU16(sdata, 18)
        self._trigUTC = self._decodeVal(sdata, 20, 4)
        self._trigUTC = self._trigUTC + (ms / 1000.0)
        recOfs = 24
        while sdata[recOfs] >= 32:
            self._var1unit = "%s%c" % (self._var1unit, sdata[recOfs])
            recOfs = recOfs + 1
        if self._var2size > 0:
            recOfs = recOfs + 1
            while sdata[recOfs] >= 32:
                self._var2unit = "%s%c" % (self._var2unit, sdata[recOfs])
                recOfs = recOfs + 1
        if self._var3size > 0:
            recOfs = recOfs + 1
            while sdata[recOfs] >= 32:
                self._var3unit = "%s%c" % (self._var3unit, sdata[recOfs])
                recOfs = recOfs + 1
        if (recOfs & 1) == 1:
            # align to next word
            recOfs = recOfs + 1
        mult1 = 1
        mult2 = 1
        mult3 = 1
        if recOfs < self._recOfs:
            # load optional value multiplier
            mult1 = self._decodeU16(sdata, recOfs)
            recOfs = recOfs + 2
            if self._var2size > 0:
                mult2 = self._decodeU16(sdata, recOfs)
                recOfs = recOfs + 2
            if self._var3size > 0:
                mult3 = self._decodeU16(sdata, recOfs)
                recOfs = recOfs + 2

        recOfs = self._recOfs
        count = self._nRecs
        while (count > 0) and(recOfs + self._var1size <= buffSize):
            v = self._decodeVal(sdata, recOfs, self._var1size) / 1000.0
            self._var1samples.append(v*mult1)
            recOfs = recOfs + recSize

        if self._var2size > 0:
            recOfs = self._recOfs + self._var1size
            count = self._nRecs
            while (count > 0) and(recOfs + self._var2size <= buffSize):
                v = self._decodeVal(sdata, recOfs, self._var2size) / 1000.0
                self._var2samples.append(v*mult2)
                recOfs = recOfs + recSize
        if self._var3size > 0:
            recOfs = self._recOfs + self._var1size + self._var2size
            count = self._nRecs
            while (count > 0) and(recOfs + self._var3size <= buffSize):
                v = self._decodeVal(sdata, recOfs, self._var3size) / 1000.0
                self._var3samples.append(v*mult3)
                recOfs = recOfs + recSize
        return YAPI.SUCCESS

    def get_serieCount(self) -> int:
        """
        Returns the number of series available in the capture.

        @return an integer corresponding to the number of
                simultaneous data series available.
        """
        return self._nVars

    def get_recordCount(self) -> int:
        """
        Returns the number of records captured (in a serie).
        In the exceptional case where it was not possible
        to transfer all data in time, the number of records
        actually present in the series might be lower than
        the number of records captured

        @return an integer corresponding to the number of
                records expected in each serie.
        """
        return self._nRecs

    def get_samplingRate(self) -> int:
        """
        Returns the effective sampling rate of the device.

        @return an integer corresponding to the number of
                samples taken each second.
        """
        return self._samplesPerSec

    def get_captureType(self) -> int:
        """
        Returns the type of automatic conditional capture
        that triggered the capture of this data sequence.

        @return the type of conditional capture.
        """
        return int(self._trigType)

    def get_triggerValue(self) -> float:
        """
        Returns the threshold value that triggered
        this automatic conditional capture, if it was
        not an instant captured triggered manually.

        @return the conditional threshold value
                at the time of capture.
        """
        return self._trigVal

    def get_triggerPosition(self) -> int:
        """
        Returns the index in the series of the sample
        corresponding to the exact time when the capture
        was triggered. In case of trigger based on average
        or RMS value, the trigger index corresponds to
        the end of the averaging period.

        @return an integer corresponding to a position
                in the data serie.
        """
        return self._trigPos

    def get_triggerRealTimeUTC(self) -> float:
        """
        Returns the absolute time when the capture was
        triggered, as a Unix timestamp. Milliseconds are
        included in this timestamp (floating-point number).

        @return a floating-point number corresponding to
                the number of seconds between the Jan 1,
                1970 and the moment where the capture
                was triggered.
        """
        return self._trigUTC

    def get_serie1Unit(self) -> str:
        """
        Returns the unit of measurement for data points in
        the first serie.

        @return a string containing to a physical unit of
                measurement.
        """
        return self._var1unit

    def get_serie2Unit(self) -> str:
        """
        Returns the unit of measurement for data points in
        the second serie.

        @return a string containing to a physical unit of
                measurement.
        """
        if not (self._nVars >= 2):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 2 in this capture data")
            return ""
        return self._var2unit

    def get_serie3Unit(self) -> str:
        """
        Returns the unit of measurement for data points in
        the third serie.

        @return a string containing to a physical unit of
                measurement.
        """
        if not (self._nVars >= 3):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 3 in this capture data")
            return ""
        return self._var3unit

    def get_serie1Values(self) -> list[float]:
        """
        Returns the sampled data corresponding to the first serie.
        The corresponding physical unit can be obtained
        using the method get_serie1Unit().

        @return a list of real numbers corresponding to all
                samples received for serie 1.

        On failure, throws an exception or returns an empty array.
        """
        return self._var1samples

    def get_serie2Values(self) -> list[float]:
        """
        Returns the sampled data corresponding to the second serie.
        The corresponding physical unit can be obtained
        using the method get_serie2Unit().

        @return a list of real numbers corresponding to all
                samples received for serie 2.

        On failure, throws an exception or returns an empty array.
        """
        if not (self._nVars >= 2):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 2 in this capture data")
            return self._var2samples
        return self._var2samples

    def get_serie3Values(self) -> list[float]:
        """
        Returns the sampled data corresponding to the third serie.
        The corresponding physical unit can be obtained
        using the method get_serie3Unit().

        @return a list of real numbers corresponding to all
                samples received for serie 3.

        On failure, throws an exception or returns an empty array.
        """
        if not (self._nVars >= 3):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 3 in this capture data")
            return self._var3samples
        return self._var3samples

    # --- (end of generated code: YInputCaptureData implementation)


# --- (generated code: YInputCapture class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YInputCaptureValueCallback = Union[Callable[['YInputCapture', str], Any], None]
    except TypeError:
        YInputCaptureValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YInputCapture(YFunction):
    """
    The YInputCapture class allows you to access data samples
    measured by a Yoctopuce electrical sensor. The data capture can be
    triggered manually, or be configured to detect specific events.

    """
    # --- (end of generated code: YInputCapture class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YInputCapture return codes)
        LASTCAPTURETIME_INVALID: Final[int] = YAPI.INVALID_LONG
        NSAMPLES_INVALID: Final[int] = YAPI.INVALID_UINT
        SAMPLINGRATE_INVALID: Final[int] = YAPI.INVALID_UINT
        CONDVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        CONDALIGN_INVALID: Final[int] = YAPI.INVALID_UINT
        CONDVALUEATSTARTUP_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        CAPTURETYPE_NONE: Final[int] = 0
        CAPTURETYPE_TIMED: Final[int] = 1
        CAPTURETYPE_V_MAX: Final[int] = 2
        CAPTURETYPE_V_MIN: Final[int] = 3
        CAPTURETYPE_I_MAX: Final[int] = 4
        CAPTURETYPE_I_MIN: Final[int] = 5
        CAPTURETYPE_P_MAX: Final[int] = 6
        CAPTURETYPE_P_MIN: Final[int] = 7
        CAPTURETYPE_V_AVG_MAX: Final[int] = 8
        CAPTURETYPE_V_AVG_MIN: Final[int] = 9
        CAPTURETYPE_V_RMS_MAX: Final[int] = 10
        CAPTURETYPE_V_RMS_MIN: Final[int] = 11
        CAPTURETYPE_I_AVG_MAX: Final[int] = 12
        CAPTURETYPE_I_AVG_MIN: Final[int] = 13
        CAPTURETYPE_I_RMS_MAX: Final[int] = 14
        CAPTURETYPE_I_RMS_MIN: Final[int] = 15
        CAPTURETYPE_P_AVG_MAX: Final[int] = 16
        CAPTURETYPE_P_AVG_MIN: Final[int] = 17
        CAPTURETYPE_PF_MIN: Final[int] = 18
        CAPTURETYPE_DPF_MIN: Final[int] = 19
        CAPTURETYPE_INVALID: Final[int] = -1
        CAPTURETYPEATSTARTUP_NONE: Final[int] = 0
        CAPTURETYPEATSTARTUP_TIMED: Final[int] = 1
        CAPTURETYPEATSTARTUP_V_MAX: Final[int] = 2
        CAPTURETYPEATSTARTUP_V_MIN: Final[int] = 3
        CAPTURETYPEATSTARTUP_I_MAX: Final[int] = 4
        CAPTURETYPEATSTARTUP_I_MIN: Final[int] = 5
        CAPTURETYPEATSTARTUP_P_MAX: Final[int] = 6
        CAPTURETYPEATSTARTUP_P_MIN: Final[int] = 7
        CAPTURETYPEATSTARTUP_V_AVG_MAX: Final[int] = 8
        CAPTURETYPEATSTARTUP_V_AVG_MIN: Final[int] = 9
        CAPTURETYPEATSTARTUP_V_RMS_MAX: Final[int] = 10
        CAPTURETYPEATSTARTUP_V_RMS_MIN: Final[int] = 11
        CAPTURETYPEATSTARTUP_I_AVG_MAX: Final[int] = 12
        CAPTURETYPEATSTARTUP_I_AVG_MIN: Final[int] = 13
        CAPTURETYPEATSTARTUP_I_RMS_MAX: Final[int] = 14
        CAPTURETYPEATSTARTUP_I_RMS_MIN: Final[int] = 15
        CAPTURETYPEATSTARTUP_P_AVG_MAX: Final[int] = 16
        CAPTURETYPEATSTARTUP_P_AVG_MIN: Final[int] = 17
        CAPTURETYPEATSTARTUP_PF_MIN: Final[int] = 18
        CAPTURETYPEATSTARTUP_DPF_MIN: Final[int] = 19
        CAPTURETYPEATSTARTUP_INVALID: Final[int] = -1
        # --- (end of generated code: YInputCapture return codes)

    # --- (generated code: YInputCapture attributes declaration)
    _valueCallback: YInputCaptureValueCallback
    # --- (end of generated code: YInputCapture attributes declaration)

    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, 'InputCapture', func)
        # --- (generated code: YInputCapture constructor)
        # --- (end of generated code: YInputCapture constructor)

    # --- (generated code: YInputCapture implementation)
    @classmethod
    def FindInputCapture(cls, func: str) -> YInputCapture:
        """
        Retrieves an instant snapshot trigger for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the instant snapshot trigger is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YInputCapture.isOnline() to test if the instant snapshot trigger is
        indeed online at a given time. In case of ambiguity when looking for
        an instant snapshot trigger by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the instant snapshot trigger, for instance
                MyDevice.inputCapture.

        @return a YInputCapture object allowing you to drive the instant snapshot trigger.
        """
        return cls.FindInputCaptureInContext(YAPI, func)

    @classmethod
    def FindInputCaptureInContext(cls, yctx: YAPIContext, func: str) -> YInputCapture:
        """
        Retrieves an instant snapshot trigger for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the instant snapshot trigger is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YInputCapture.isOnline() to test if the instant snapshot trigger is
        indeed online at a given time. In case of ambiguity when looking for
        an instant snapshot trigger by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the instant snapshot trigger, for instance
                MyDevice.inputCapture.

        @return a YInputCapture object allowing you to drive the instant snapshot trigger.
        """
        obj: Union[YInputCapture, None] = yctx._findInCache('InputCapture', func)
        if obj:
            return obj
        return YInputCapture(yctx, func)

    @classmethod
    def FirstInputCapture(cls) -> Union[YInputCapture, None]:
        """
        Starts the enumeration of instant snapshot triggers currently accessible.
        Use the method YInputCapture.nextInputCapture() to iterate on
        next instant snapshot triggers.

        @return a pointer to a YInputCapture object, corresponding to
                the first instant snapshot trigger currently online, or a None pointer
                if there are none.
        """
        return cls.FirstInputCaptureInContext(YAPI)

    @classmethod
    def FirstInputCaptureInContext(cls, yctx: YAPIContext) -> Union[YInputCapture, None]:
        """
        Starts the enumeration of instant snapshot triggers currently accessible.
        Use the method YInputCapture.nextInputCapture() to iterate on
        next instant snapshot triggers.

        @param yctx : a YAPI context.

        @return a pointer to a YInputCapture object, corresponding to
                the first instant snapshot trigger currently online, or a None pointer
                if there are none.
        """
        hwid: Union[HwId, None] = yctx._firstHwId('InputCapture')
        if hwid:
            return cls.FindInputCaptureInContext(yctx, hwid2str(hwid))
        return None

    def nextInputCapture(self) -> Union[YInputCapture, None]:
        """
        Continues the enumeration of instant snapshot triggers started using yFirstInputCapture().
        Caution: You can't make any assumption about the returned instant snapshot triggers order.
        If you want to find a specific an instant snapshot trigger, use InputCapture.findInputCapture()
        and a hardwareID or a logical name.

        @return a pointer to a YInputCapture object, corresponding to
                an instant snapshot trigger currently online, or a None pointer
                if there are no more instant snapshot triggers to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            next_hwid = self._yapi._nextHwId('InputCapture', self.get_hwId())
        except YAPI_Exception:
            pass
        if next_hwid:
            return self.FindInputCaptureInContext(self._yapi, hwid2str(next_hwid))
        return None

    async def get_lastCaptureTime(self) -> int:
        """
        Returns the number of elapsed milliseconds between the module power on
        and the last capture (time of trigger), or zero if no capture has been done.

        @return an integer corresponding to the number of elapsed milliseconds between the module power on
                and the last capture (time of trigger), or zero if no capture has been done

        On failure, throws an exception or returns YInputCapture.LASTCAPTURETIME_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("lastCaptureTime")
        if json_val is None:
            return YInputCapture.LASTCAPTURETIME_INVALID
        return json_val

    async def get_nSamples(self) -> int:
        """
        Returns the number of samples that will be captured.

        @return an integer corresponding to the number of samples that will be captured

        On failure, throws an exception or returns YInputCapture.NSAMPLES_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("nSamples")
        if json_val is None:
            return YInputCapture.NSAMPLES_INVALID
        return json_val

    async def set_nSamples(self, newval: int) -> int:
        """
        Changes the type of automatic conditional capture.
        The maximum number of samples depends on the device memory.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : an integer corresponding to the type of automatic conditional capture

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("nSamples", rest_val)

    async def get_samplingRate(self) -> int:
        """
        Returns the sampling frequency, in Hz.

        @return an integer corresponding to the sampling frequency, in Hz

        On failure, throws an exception or returns YInputCapture.SAMPLINGRATE_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("samplingRate")
        if json_val is None:
            return YInputCapture.SAMPLINGRATE_INVALID
        return json_val

    async def get_captureType(self) -> int:
        """
        Returns the type of automatic conditional capture.

        @return a value among YInputCapture.CAPTURETYPE_NONE, YInputCapture.CAPTURETYPE_TIMED,
        YInputCapture.CAPTURETYPE_V_MAX, YInputCapture.CAPTURETYPE_V_MIN, YInputCapture.CAPTURETYPE_I_MAX,
        YInputCapture.CAPTURETYPE_I_MIN, YInputCapture.CAPTURETYPE_P_MAX, YInputCapture.CAPTURETYPE_P_MIN,
        YInputCapture.CAPTURETYPE_V_AVG_MAX, YInputCapture.CAPTURETYPE_V_AVG_MIN,
        YInputCapture.CAPTURETYPE_V_RMS_MAX, YInputCapture.CAPTURETYPE_V_RMS_MIN,
        YInputCapture.CAPTURETYPE_I_AVG_MAX, YInputCapture.CAPTURETYPE_I_AVG_MIN,
        YInputCapture.CAPTURETYPE_I_RMS_MAX, YInputCapture.CAPTURETYPE_I_RMS_MIN,
        YInputCapture.CAPTURETYPE_P_AVG_MAX, YInputCapture.CAPTURETYPE_P_AVG_MIN,
        YInputCapture.CAPTURETYPE_PF_MIN and YInputCapture.CAPTURETYPE_DPF_MIN corresponding to the type of
        automatic conditional capture

        On failure, throws an exception or returns YInputCapture.CAPTURETYPE_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("captureType")
        if json_val is None:
            return YInputCapture.CAPTURETYPE_INVALID
        return json_val

    async def set_captureType(self, newval: int) -> int:
        """
        Changes the type of automatic conditional capture.

        @param newval : a value among YInputCapture.CAPTURETYPE_NONE, YInputCapture.CAPTURETYPE_TIMED,
        YInputCapture.CAPTURETYPE_V_MAX, YInputCapture.CAPTURETYPE_V_MIN, YInputCapture.CAPTURETYPE_I_MAX,
        YInputCapture.CAPTURETYPE_I_MIN, YInputCapture.CAPTURETYPE_P_MAX, YInputCapture.CAPTURETYPE_P_MIN,
        YInputCapture.CAPTURETYPE_V_AVG_MAX, YInputCapture.CAPTURETYPE_V_AVG_MIN,
        YInputCapture.CAPTURETYPE_V_RMS_MAX, YInputCapture.CAPTURETYPE_V_RMS_MIN,
        YInputCapture.CAPTURETYPE_I_AVG_MAX, YInputCapture.CAPTURETYPE_I_AVG_MIN,
        YInputCapture.CAPTURETYPE_I_RMS_MAX, YInputCapture.CAPTURETYPE_I_RMS_MIN,
        YInputCapture.CAPTURETYPE_P_AVG_MAX, YInputCapture.CAPTURETYPE_P_AVG_MIN,
        YInputCapture.CAPTURETYPE_PF_MIN and YInputCapture.CAPTURETYPE_DPF_MIN corresponding to the type of
        automatic conditional capture

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("captureType", rest_val)

    async def set_condValue(self, newval: float) -> int:
        """
        Changes current threshold value for automatic conditional capture.

        @param newval : a floating point number corresponding to current threshold value for automatic
        conditional capture

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return await self._setAttr("condValue", rest_val)

    async def get_condValue(self) -> float:
        """
        Returns current threshold value for automatic conditional capture.

        @return a floating point number corresponding to current threshold value for automatic conditional capture

        On failure, throws an exception or returns YInputCapture.CONDVALUE_INVALID.
        """
        json_val: Union[float, None] = await self._fromCache("condValue")
        if json_val is None:
            return YInputCapture.CONDVALUE_INVALID
        return round(json_val / 65.536) / 1000.0

    async def get_condAlign(self) -> int:
        """
        Returns the relative position of the trigger event within the capture window.
        When the value is 50%, the capture is centered on the event.

        @return an integer corresponding to the relative position of the trigger event within the capture window

        On failure, throws an exception or returns YInputCapture.CONDALIGN_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("condAlign")
        if json_val is None:
            return YInputCapture.CONDALIGN_INVALID
        return json_val

    async def set_condAlign(self, newval: int) -> int:
        """
        Changes the relative position of the trigger event within the capture window.
        The new value must be between 10% (on the left) and 90% (on the right).
        When the value is 50%, the capture is centered on the event.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : an integer corresponding to the relative position of the trigger event within the capture window

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("condAlign", rest_val)

    async def get_captureTypeAtStartup(self) -> int:
        """
        Returns the type of automatic conditional capture
        applied at device power on.

        @return a value among YInputCapture.CAPTURETYPEATSTARTUP_NONE,
        YInputCapture.CAPTURETYPEATSTARTUP_TIMED, YInputCapture.CAPTURETYPEATSTARTUP_V_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_PF_MIN and
        YInputCapture.CAPTURETYPEATSTARTUP_DPF_MIN corresponding to the type of automatic conditional capture
                applied at device power on

        On failure, throws an exception or returns YInputCapture.CAPTURETYPEATSTARTUP_INVALID.
        """
        json_val: Union[int, None] = await self._fromCache("captureTypeAtStartup")
        if json_val is None:
            return YInputCapture.CAPTURETYPEATSTARTUP_INVALID
        return json_val

    async def set_captureTypeAtStartup(self, newval: int) -> int:
        """
        Changes the type of automatic conditional capture
        applied at device power on.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : a value among YInputCapture.CAPTURETYPEATSTARTUP_NONE,
        YInputCapture.CAPTURETYPEATSTARTUP_TIMED, YInputCapture.CAPTURETYPEATSTARTUP_V_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_PF_MIN and
        YInputCapture.CAPTURETYPEATSTARTUP_DPF_MIN corresponding to the type of automatic conditional capture
                applied at device power on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("captureTypeAtStartup", rest_val)

    async def set_condValueAtStartup(self, newval: float) -> int:
        """
        Changes current threshold value for automatic conditional
        capture applied at device power on.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : a floating point number corresponding to current threshold value for automatic conditional
                capture applied at device power on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return await self._setAttr("condValueAtStartup", rest_val)

    async def get_condValueAtStartup(self) -> float:
        """
        Returns the threshold value for automatic conditional
        capture applied at device power on.

        @return a floating point number corresponding to the threshold value for automatic conditional
                capture applied at device power on

        On failure, throws an exception or returns YInputCapture.CONDVALUEATSTARTUP_INVALID.
        """
        json_val: Union[float, None] = await self._fromCache("condValueAtStartup")
        if json_val is None:
            return YInputCapture.CONDVALUEATSTARTUP_INVALID
        return round(json_val / 65.536) / 1000.0

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YInputCaptureValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is called once when it is registered, passing the current advertised value
            of the function, provided that it is not an empty string.
            The callback is then invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    async def get_lastCapture(self) -> YInputCaptureData:
        """
        Returns all details about the last automatic input capture.

        @return an YInputCaptureData object including
                data series and all related meta-information.
                On failure, throws an exception or returns an capture object.
        """
        snapData: xarray

        snapData = await self._download("snap.bin")
        return YInputCaptureData(self, snapData)

    async def get_immediateCapture(self, msDuration: int) -> YInputCaptureData:
        """
        Returns a new immediate capture of the device inputs.

        @param msDuration : duration of the capture window,
                in milliseconds (eg. between 20 and 1000).

        @return an YInputCaptureData object including
                data series for the specified duration.
                On failure, throws an exception or returns an capture object.
        """
        snapUrl: str
        snapData: xarray
        snapStart: int
        if msDuration < 1:
            msDuration = 20
        if msDuration > 1000:
            msDuration = 1000
        snapStart = -msDuration // 2
        snapUrl = "snap.bin?t=%d&d=%d" % (snapStart, msDuration)

        snapData = await self._download(snapUrl)
        return YInputCaptureData(self, snapData)

    # --- (end of generated code: YInputCapture implementation)
